﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ParsecParser;
using System.Xml.Linq;
using System.IO;
using System.Deployment.Application;
using ParsecClient.ParsecRaidWebService;

namespace ParsecClient.Classes
{
    [Serializable]
    public class RaidDataFile
    {
        public List<RaidData> RaidFights { get; set; }
    }
    [Serializable]
    public class RaidData
    {
        public static string RaidDataFileNamePrefix = "RaidData_";
        public int FightIndex { get; set; }
        public List<CombatStatistics> RaidDetails { get; set; }
        public RaidData()
        {
            RaidDetails = new List<CombatStatistics>();
        }
        public static void ArchiveRaidData(string logFileName, RaidData data)
        {
            if (string.IsNullOrEmpty(logFileName))
                throw new ArgumentException("parameter logFileName is null or empty", "logFileName");
            string folder;
            if (ApplicationDeployment.IsNetworkDeployed)
            {
                folder = ApplicationDeployment.CurrentDeployment.DataDirectory;
            }
            else
            {
                folder = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "Parsec");
                if (!Directory.Exists(folder))
                {
                    Directory.CreateDirectory(folder);
                }
            }
            string fileName = Path.ChangeExtension(RaidDataFileNamePrefix + logFileName, "xml");
            string fullPath = Path.Combine(folder, fileName);

            //If file doesnt exist
            if (!File.Exists(fullPath))
            {
                RaidDataFile rdf = new RaidDataFile() { RaidFights = new List<RaidData>() {data} };
                System.Xml.Serialization.XmlSerializer writer = new System.Xml.Serialization.XmlSerializer(typeof(RaidDataFile));
                System.IO.StreamWriter file = new System.IO.StreamWriter(fullPath);
                writer.Serialize(file, rdf);
                file.Close();
            }
            else
            {
                //load file
                System.Xml.Serialization.XmlSerializer reader = new System.Xml.Serialization.XmlSerializer(typeof(RaidDataFile));
                System.IO.StreamReader freader = new System.IO.StreamReader(fullPath);
                RaidDataFile rdf = (RaidDataFile)reader.Deserialize(freader);
                freader.Close();
                //if fight index exists update data
                rdf.RaidFights.RemoveAll(x => x.FightIndex == data.FightIndex);
                rdf.RaidFights.Add(data);
                System.Xml.Serialization.XmlSerializer writer = new System.Xml.Serialization.XmlSerializer(typeof(RaidDataFile));
                System.IO.StreamWriter fwriter = new System.IO.StreamWriter(fullPath);
                writer.Serialize(fwriter, rdf);
                freader.Close();
                //else append xml with new data
            }
        }
    }
}
