﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ParsecParser;
using System.IO;
using Microsoft.Win32;
using ParsecClient.Classes;
using System.Diagnostics;
using ParsecClient.ParsecRaidWebService;
using System.Configuration;

namespace ParsecClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Constants
        private const string DefaultLogPath = "Star Wars - The Old Republic\\CombatLogs\\";
        private const int MaxConsecutiveLogReadErrors = 3;
        private const int MaxConsecutiveRaidSyncErrors = 3;
        private const int MaxConsecutiveReadsWithNoData = 3;
        private const int MaxOutOfCombatRaidSyncs = 2;
        private const int MaxRaidSyncWithNoUpdate = 2;
        private const int RaidUpdateHeartBeatSeconds = 30;
        private const int RaidTimeoutSeconds = 60 * 45;
        public const string AppVersion = "1.0.0.20";
        private const int PollingRate = 3000;
        #endregion

        #region Properties
        private string LastOpenedLogFolder;
        private RealTimeLogReader LogReader;
        private Parser LogParser;
        public CombatLog Log;
        public string PreviousLoggedCharacter;
        public string LogFileFullPath;
        private bool IsWatching = false;
        public string LogFolderPath;
        private CombatLogFilterParameters LogFilterParameters = new CombatLogFilterParameters() { ShowAll = true };
        private StatsOverview StatPopOut;
        private PvpStatsOverview PvpStatsPopOut;
        private RaidDps RaidDpsPopOut;
        private RaidHps RaidHpsPopOut;
        private RaidTps RaidTpsPopOut;
        private Window PopUpHost;
        private MessageWindow MyMessageWindow;
        private int ConsecutiveLogReadErrors = 0;
        private int ConsecutiveRaidSyncErrors = 0;
        private int ConsecutiveOutOfCombatRaidSyncs = 0;
        private int ConsecutiveRaidHeartBeatWithNoUpdate = 0;
        private int ConsecutiveReadsWithNoData = 0;
        private bool IsRaidPolling = false;
        public List<RaidGroup> RaidGroups = new List<RaidGroup>();
        public RaidGroup CurrentRaidGroup = new RaidGroup();
        public List<string> ConnectedRaiders = new List<string>();
        public DateTime? RaidLastUpdatedDate;
        public DateTime LastCombatRaidSync;
        public bool IsLastFightSelected = true;
        public DateTime LastRaidSyncDate;
        public List<string> RecentFiles;
        public bool IsStreamMode = false;
        public ParsecRaidServiceSoapClient RaidService = new ParsecRaidServiceSoapClient();
        //public string AlternateServiceUrl;
        //private Random MyRandom = new Random();
        public bool IsArchiveEnabled = false;
        #endregion
        
        #region Constructors
        public MainWindow()
        {
            InitializeComponent();//System.Reflection.Assembly.GetEntryAssembly().GetName().Version.ToString();
            try
            {
                Logger.WriteInfo("Parsec Startup " + AppVersion);
                RecentFiles = new List<string>();
                CommandBindings.Add(new CommandBinding(ApplicationCommands.Close, new ExecutedRoutedEventHandler(delegate(object sender, ExecutedRoutedEventArgs args) { Application.Current.Shutdown(); })));
                rdoThisFight.IsChecked = true;
                rdoThisFight.Checked += FightTypeChangedHandler;
                rdoAllFights.Checked += FightTypeChangedHandler;
                StatPopOut = new StatsOverview(this);
                RaidDpsPopOut = new RaidDps(this);
                RaidHpsPopOut = new RaidHps(this);
                RaidTpsPopOut = new RaidTps(this);
                PvpStatsPopOut = new PvpStatsOverview(this);
                PopUpHost = new Window();
                PopUpHost.Top = -100;
                PopUpHost.Left = -100;
                PopUpHost.Width = 1;
                PopUpHost.Height = 1;
                PopUpHost.WindowStyle = WindowStyle.ToolWindow;
                PopUpHost.Show();
                PopUpHost.Hide();
                bool showPing = Convert.ToBoolean(ConfigurationManager.AppSettings["ShowPing"]);
                IsArchiveEnabled = Convert.ToBoolean(ConfigurationManager.AppSettings["EnableDataArchive"]);
                txtSyncMs.Visibility = showPing ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed;
                //AlternateServiceUrl = ConfigurationManager.AppSettings["AlternateServiceUrl"];
                try
                {
                    LoadSettings();
                    BindRaidGroups();
                    BindRecentFiles();
                }
                catch (Exception ex)
                {
                    //write to log
                    HandleError(null, "Settings Error, Attempting Reset!", ex, false);
                    ResetSettings(true);
                }

                if (IsStreamMode)
                {
                    SetStreamMode(this, true, WhiteBackground);
                    //do any pops
                    SetStreamMode(StatPopOut, true, BlackBackground);
                    SetStreamMode(PvpStatsPopOut, true, BlackBackground);
                    SetStreamMode(RaidDpsPopOut, true, BlackBackground);
                    SetStreamMode(RaidHpsPopOut, true, BlackBackground);
                    SetStreamMode(RaidTpsPopOut, true, BlackBackground);
                }
            }
            catch (Exception ex2)
            {
                try
                {

                    HandleError(null, "Constructor Error", ex2, false);
                }
                catch 
                {
                    //swallow error on logging
                }
            }
        }
        #endregion

        #region Event Handlers
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            string openFilePath = null;
            try
            {
                //Debug.WriteLine("Window Loaded");
                //check for commandline args
                bool resetSettings = false;
                string[] args = Environment.GetCommandLineArgs();
                for (int index = 1; index < args.Length; index ++)
                {
                    string argString = args[index];
                    if (!string.IsNullOrEmpty(argString.Trim()))
                    {
                        if (string.Compare(argString, "/resetsettings", true) == 1)
                        {
                            resetSettings = true;
                        }
                        else if (argString.IndexOfAny(System.IO.Path.GetInvalidPathChars()) == -1)
                        {
                            try
                            {
                                if (File.Exists(argString) && string.Compare(System.IO.Path.GetExtension(argString), ".txt", true) == 0)
                                {
                                    openFilePath = argString;
                                    break;
                                }
                            }
                            catch
                            {
                                //do nothing
                            }
                        }
                    }
                }
                chkShowAll.IsChecked = true;
                if (resetSettings)
                {
                    ResetSettings(false);
                }
                if (Properties.Settings.Default.LastRunVersion != AppVersion)
                {
                    ShowNotes();
                    Properties.Settings.Default.LastRunVersion = AppVersion;
                    Properties.Settings.Default.Save();
                }

            }
            catch (Exception ex)
            {
                HandleError(null, "Error on Window_Loaded Event", ex, true);
            }


            try
            {
                if (!string.IsNullOrEmpty(openFilePath))
                {
                    LogFileFullPath = openFilePath;
                    ProcessLogFile(true);
                    AddRecentFile(openFilePath);
                    BindRecentFiles();
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Loading File", "Error on Window_Loaded FileOpen", ex, true);
            }
        }
        private void btnWatch_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                //Debug.WriteLine("Watch button Clicked");
                if (!IsWatching)
                {
                    LogFileFullPath = RealTimeLogReader.GetNewestFile(LogFolderPath);
                    if (string.IsNullOrEmpty(LogFileFullPath))
                        throw new Exception("Log File Not Found");
                    if (string.IsNullOrEmpty(LogFolderPath))
                        throw new Exception("Log Folder Does not exist");
                    if (!Directory.Exists(LogFolderPath))
                        throw new Exception("Log Folder Does Not Exist");
                    if (!File.Exists(LogFileFullPath))
                        throw new Exception("Log Folder Does Not Exist");
                    StartWatching();
                    btnWatch.Visibility = System.Windows.Visibility.Collapsed;
                    btnStop.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    StopWatching();
                    btnStop.Visibility = System.Windows.Visibility.Collapsed;
                    btnWatch.Visibility = System.Windows.Visibility.Visible;
                    if (IsRaidPolling)
                    {
                        DisconnectFromRaid(Log.CharacterName);
                        SetDisconnected();
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Watching File", "Error on btnWatch Click Event", ex, true);
            }
        }
        private void LogUpdatedHandler(object sender, OnLogUpdatedEventArgs e)
        {

            if (!this.Dispatcher.CheckAccess())
            {
                this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, (Action)delegate { LogUpdatedHandler(sender, e); });
                return;
            }
            else
            {
                bool isNewFight = false;
                try
                {
                    //Debug.WriteLine("Log Update Event Detected");
                    MovePopOutsToFront();
                    if (e.NewLines.Count > 0)
                    {
                        ConsecutiveReadsWithNoData = 0;
                        int previousLastFightIndex = Log.LastFightIndex;
                        List<CombatLogEvent> newEvents = Log.AddLines(e.NewLines).Where(x => x.FightIndex == Log.LastFightIndex).ToList();
                        UpdateCharacterName();
                        if (Log.LastFightIndex > 0 && newEvents.Count > 0)
                        {
                            isNewFight = Log.LastFightIndex > previousLastFightIndex;
                            if (isNewFight)//detect a new fight and then add it to list but reselect current fight when finished
                            {
                                BindFightList(!IsLastFightSelected, Log.LastFightIndex, false);
                            }
                            //if fight ended and none started update last time
                            if (IsLastFightSelected)
                            {
                                //update interface if currentFight is selected
                                CombatDetails currentFight = Log.Fights[Log.LastFightIndex];
                                UpdateStats(currentFight, Log.LogStatistics);
                                BindFightTabs(currentFight);
                                BindCombatLog(newEvents, isNewFight);
                            }
                            PvpStatsPopOut.SetStats(newEvents);
                        }
                        ConsecutiveLogReadErrors = 0;
                    }
                    else
                    {
                        //pulse with no lines
                        if (ConsecutiveReadsWithNoData > MaxConsecutiveReadsWithNoData)
                            Log.EndCurrentFight();
                        else
                            ConsecutiveReadsWithNoData++;
                    }
                }
                catch (Exception ex)
                {
                    ConsecutiveLogReadErrors++;
                    if (ConsecutiveLogReadErrors > MaxConsecutiveLogReadErrors)
                    {
                        btnWatch_Click(new object(), new RoutedEventArgs()); //stop watching
                        HandleError("Parsec is experiencing errors parsing your log file", string.Format("Error on LogUpdatedHandler consecutive:{0}", ConsecutiveLogReadErrors), ex, false);
                    }
                    else
                    {
                        HandleError(null, string.Format("Error on LogUpdatedHandler consecutive:{0}", ConsecutiveLogReadErrors), ex, false);
                    }
                }
                try
                {
                    if (IsRaidPolling)
                    {
                        if (!string.IsNullOrEmpty(Log.CharacterName))
                        {
                            if (Log.IsInCombat)
                                ConsecutiveOutOfCombatRaidSyncs = 0;
                            bool syncWithServer = false;
                            bool sendData = false;
                            if (ConsecutiveOutOfCombatRaidSyncs < MaxOutOfCombatRaidSyncs)
                            {
                                ConsecutiveOutOfCombatRaidSyncs++;
                                syncWithServer = true;
                                sendData = true;

                            }
                            else if (RaidLastUpdatedDate.HasValue && ConsecutiveRaidHeartBeatWithNoUpdate < MaxRaidSyncWithNoUpdate)
                            {
                                syncWithServer = true;
                                sendData = false;
                            }
                            else if (DateTime.UtcNow.Subtract(LastRaidSyncDate).TotalSeconds > RaidUpdateHeartBeatSeconds)
                            {
                                if (DateTime.UtcNow.Subtract(LastCombatRaidSync).TotalSeconds > RaidTimeoutSeconds)
                                {
                                    //disconnect
                                    DisconnectFromRaid(Log.CharacterName);
                                    SetDisconnected();
                                    HandleError("You have been disconnected from the raid due to inactivity", null, null, false);
                                }
                                else
                                {
                                    syncWithServer = true;
                                    sendData = false;
                                    //sync names only
                                }
                            }
                            if (syncWithServer)
                            {
                                CombatDetails currentFight = new CombatDetails();
                                if (Log.LastFightIndex > 0)
                                {
                                    currentFight = Log.Fights[Log.LastFightIndex];
                                }
                                int requestedPollRate = PollingRate;
                                SendRaidServerHeartBeat(currentFight, sendData, ref requestedPollRate);
                                if (sendData)//meh?
                                {
                                    if (Log.LastFightIndex > 0)
                                    {
                                        if (IsLastFightSelected)
                                        {
                                            BindRaidDetails(currentFight);
                                        }
                                    }
                                }
                                BindConnectedRaiders(!IsLastFightSelected);
                                ConsecutiveRaidSyncErrors = 0;
                                if (requestedPollRate != LogReader.PollMilliseconds)
                                {
                                    bool wasWatching = IsWatching;
                                    if (IsWatching)
                                    {
                                        StopWatching(); //stop Watching
                                    }
                                    //change filename
                                    LogReader.PollMilliseconds = requestedPollRate;
                                    //if was watching start it again
                                    if (wasWatching)
                                    {
                                        StartWatching(); //start Watching
                                    }
                                }
                            }
                        }
                        else
                        {
                            DisconnectFromRaid(PreviousLoggedCharacter);
                        }
                    }
                }
                catch (Exception ex)
                {
                    ConsecutiveRaidSyncErrors++;
                    if (ConsecutiveRaidSyncErrors >= MaxConsecutiveRaidSyncErrors)
                    {
                        SetDisconnected();
                        HandleError("Parsec is experiencing errors connecting to the raid service", string.Format("Error on RaidTimer_Elapsed consecutive:{0}", ConsecutiveRaidSyncErrors), ex, false);
                    }
                    else
                    {
                        HandleError(null, string.Format("Error on LogUpdatedHandler(RaidSection) consecutive:{0}", ConsecutiveRaidSyncErrors), ex, false);
                    }
                }
                try
                {
                    if (IsArchiveEnabled && isNewFight && Log.LastFightIndex > 0 && IsRaidPolling)
                    {
                        CombatDetails archivedFight = Log.Fights[Log.LastFightIndex-1];
                        if (archivedFight.RaidDetails.Count > 0)
                        {
                            //archive raid
                            RaidData.ArchiveRaidData(Log.FileName, new RaidData() { 
                                FightIndex = Log.LastFightIndex - 1, RaidDetails = archivedFight.RaidDetails });
                        }
                    }
                }
                catch (Exception ex)
                {
                    HandleError(null, "Error on LogUpdatedHandler(ArchiveSection)", ex, false);
                }
            }
        }
        private void NewLogCreatedHandler(object sender, OnNewLogCreatedEventArgs e)
        {
            if (!this.Dispatcher.CheckAccess())
            {
                this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, (Action)delegate { NewLogCreatedHandler(sender, e); });
                return;
            }
            else
            {
                try
                {
                    //Debug.WriteLine("New Log Created Event Detected");
                    //stop watching
                    bool wasWatching = IsWatching;
                    if (IsWatching)
                    {
                        StopWatching(); //stop Watching
                    }
                    //change filename
                    LogFileFullPath = e.FileName;
                    //if was watching start it again
                    if (wasWatching)
                    {
                        StartWatching(); //start Watching
                    }
                    ConsecutiveLogReadErrors = 0;
                }
                catch (Exception ex)
                {
                    ConsecutiveLogReadErrors++;
                    if (ConsecutiveLogReadErrors > MaxConsecutiveLogReadErrors)
                    {
                        btnWatch_Click(new object(), new RoutedEventArgs()); //stop watching
                        HandleError("Parsec is experiencing errors parsing your log file", string.Format("Error on NewLogCreatedHandler consecutive:{0}", ConsecutiveLogReadErrors), ex, false);
                    }
                    else
                    {
                        HandleError(null, string.Format("Error on NewLogCreatedHandler consecutive:{0}", ConsecutiveLogReadErrors), ex, false);
                    }
                }
            }
        }
        private void LogReaderError(object sender, ErrorEventArgs e)
        {
            if (!this.Dispatcher.CheckAccess())
            {
                this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, (Action)delegate { LogReaderError(sender, e); });
                return;
            }
            else
            {                
                if (ConsecutiveLogReadErrors > MaxConsecutiveLogReadErrors)
                {
                    btnWatch_Click(new object(), new RoutedEventArgs()); //stop watching
                    HandleError("Parsec is experiencing errors parsing your log file", string.Format("Error on LogReaderError consecutive:{0}", ConsecutiveLogReadErrors), e.GetException(), false);
                }
                else
                {
                    HandleError(null, string.Format("Error on LogReaderError consecutive:{0}", ConsecutiveLogReadErrors), e.GetException(), false);
                }
            }
        }
        private void ddlFights_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                if (ddlFights.SelectedValue != null)
                {
                    int newSelectedFightIndex = Convert.ToInt32(ddlFights.SelectedValue);
                    int lastPossibleFightIndex = ddlFights.Items.Cast<FightDisplayItem>().Select(x => x.FightIndex).Max();
                    IsLastFightSelected = newSelectedFightIndex == lastPossibleFightIndex;
                    if (newSelectedFightIndex > 0)
                    {
                        CombatDetails currentFight = Log.Fights[newSelectedFightIndex];
                        UpdateStats(currentFight, Log.LogStatistics);
                        BindFightTabs(currentFight);
                        BindCombatLog(currentFight.Events, true);
                        BindRaidDetails(currentFight);
                        BindConnectedRaiders(!(IsLastFightSelected && IsWatching));
                    }
                }
                else
                {
                    IsLastFightSelected = true;
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Loading Fight", "Error on ddlFights_SelectionChanged", ex, true);
            }
        }
        private void btnLoad_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                //Debug.WriteLine("Load Button Clicked");
                OpenFileDialog openFileDialog = new OpenFileDialog();
                openFileDialog.CustomPlaces.Add(new FileDialogCustomPlace(LogFolderPath));
                if (string.IsNullOrEmpty(LastOpenedLogFolder))
                    openFileDialog.InitialDirectory = LogFolderPath;
                else
                    openFileDialog.InitialDirectory = LastOpenedLogFolder;
                openFileDialog.Filter = "Text files (*.txt)|*.txt";
                bool? result = openFileDialog.ShowDialog();
                if (result.HasValue && result.Value)
                {
                    LastOpenedLogFolder = System.IO.Path.GetDirectoryName(openFileDialog.FileName);
                    //stop watching
                    if (IsWatching)
                    {
                        StopWatching();
                        btnStop.Visibility = System.Windows.Visibility.Collapsed;
                        btnWatch.Visibility = System.Windows.Visibility.Visible;
                        if (IsRaidPolling)
                        {
                            DisconnectFromRaid(Log.CharacterName);
                            SetDisconnected();
                        }
                    }
                    //change filename
                    LogFileFullPath = openFileDialog.FileName;
                    ProcessLogFile(true);
                    AddRecentFile(LogFileFullPath);
                    BindRecentFiles();
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Loading File", "Error on btnLoad_Click", ex, true);
            }
        }
        private void btnRecentFile_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                string fpath = (string)((MenuItem)sender).ToolTip;
                if (!string.IsNullOrEmpty(fpath))
                {
                    //stop watching
                    if (IsWatching)
                    {
                        StopWatching();
                        btnStop.Visibility = System.Windows.Visibility.Collapsed;
                        btnWatch.Visibility = System.Windows.Visibility.Visible;
                        if (IsRaidPolling)
                        {
                            DisconnectFromRaid(Log.CharacterName);
                            SetDisconnected();
                        }
                    }
                    //change filename
                    LogFileFullPath = fpath;
                    ProcessLogFile(true);
                    AddRecentFile(LogFileFullPath);
                    BindRecentFiles();
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Loading File", "Error on btnLoad_Click", ex, true);
            }
        }
        private void chkShowAll_Checked(object sender, RoutedEventArgs e)
        {
            try
            {
                chkOthersEffects.IsChecked = false;
                chkAbilities.IsChecked = false;
                chkIncomingDamage.IsChecked = false;
                chkIncomingHeals.IsChecked = false;
                chkOutgoingDamage.IsChecked = false;
                chkCharactersEffects.IsChecked = false;
                chkEvents.IsChecked = false;
                chkOutgoingHeals.IsChecked = false;

                chkCharactersEffects.IsEnabled = false;
                chkAbilities.IsEnabled = false;
                chkIncomingDamage.IsEnabled = false;
                chkIncomingHeals.IsEnabled = false;
                chkOutgoingDamage.IsEnabled = false;
                chkOthersEffects.IsEnabled = false;
                chkEvents.IsEnabled = false;
                chkOutgoingHeals.IsEnabled = false;
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on chkShowAll_Checked", ex, true);
            }
        }
        private void chkShowAll_Unchecked(object sender, RoutedEventArgs e)
        {
            try
            {
                chkOthersEffects.IsEnabled = true;
                chkAbilities.IsEnabled = true;
                chkIncomingDamage.IsEnabled = true;
                chkIncomingHeals.IsEnabled = true;
                chkOutgoingDamage.IsEnabled = true;
                chkCharactersEffects.IsEnabled = true;
                chkEvents.IsEnabled = true;
                chkOutgoingHeals.IsEnabled = true;
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on chkShowAll_Unchecked", ex, true);
            }
        }
        private void btnApplyFilter_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                //Debug.WriteLine("Apply Filter Button Clicked");
                LogFilterParameters = new CombatLogFilterParameters()
                {
                    ShowAll = chkShowAll.IsChecked.Value,
                    ShowAbilities = chkAbilities.IsChecked.Value,
                    ShowOthersEffects = chkOthersEffects.IsChecked.Value,
                    ShowIncomingDamage = chkIncomingDamage.IsChecked.Value,
                    ShowIncomingHeals = chkIncomingHeals.IsChecked.Value,
                    ShowCharacterEffects = chkCharactersEffects.IsChecked.Value,
                    ShowEvents = chkEvents.IsChecked.Value,
                    ShowOutgoingDamage = chkOutgoingDamage.IsChecked.Value,
                    ShowOutgoingHeals = chkOutgoingHeals.IsChecked.Value,
                    KeyWordFilter = txtKeyWord.Text
                };
                int newSelectedFightIndex = Convert.ToInt32(ddlFights.SelectedValue);
                if (newSelectedFightIndex > 0)
                {
                    List<CombatLogEvent> existingEvents = Log.Fights[newSelectedFightIndex].Events;
                    BindCombatLog(existingEvents, true);
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Applying Filter", "Error on btnApplyFilter_Click", ex, true);
            }
        }
        private void txtKeyWord_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                if (e.Key == Key.Return)
                {
                    btnApplyFilter_Click(sender, e);
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on txtKeyWord_KeyDown", ex, true);
            }
        }
        private void btnClearKeyWord_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                txtKeyWord.Text = "";
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnClearKeyWord_Click", ex, true);
            }
        }
        private void btnFilters_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (pnlFilters.Visibility == System.Windows.Visibility.Collapsed)
                {
                    pnlFilters.Visibility = System.Windows.Visibility.Visible;
                    btnFilterExpand.Visibility = System.Windows.Visibility.Collapsed;
                    btnFilterCollapse.Visibility = System.Windows.Visibility.Visible;
                }
                else
                {
                    pnlFilters.Visibility = System.Windows.Visibility.Collapsed;
                    btnFilterExpand.Visibility = System.Windows.Visibility.Visible;
                    btnFilterCollapse.Visibility = System.Windows.Visibility.Collapsed;
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Changing Filter", "Error on btnFilters_Click", ex, true);
            }
        }
        private void txtFindWord_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                if (e.Key == Key.Return)
                {
                    btnFind_Click(sender, e);
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on txtFindWord_KeyDown", ex, true);
            }
        }
        private void btnClearFindWord_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                txtFindWord.Text = "";
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnClearFindWord_Click", ex, true);
            }
        }
        private void btnFind_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.MyCombatLogControl.Find(txtFindWord.Text);
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnFind_Click", ex, true);
            }
        }
        private void btnFindNext_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.MyCombatLogControl.FindNext(txtFindWord.Text);
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnFindNext_Click", ex, true);
            }
        }
        private void btnFindPrevious_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.MyCombatLogControl.FindPrevious(txtFindWord.Text);
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnFindPrevious_Click", ex, true);
            }
        }
        private void btnExit_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Application.Current.Shutdown();
            }
            catch (Exception ex)
            {
                Logger.WriteError("Error on ShutDown", ex);
                HandleError(null, "Error on ShutDown", ex, true);
            }
        }
        private void btnSettings_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                SaveSettings();
                SettingsWindow s = new SettingsWindow();
                s.Owner = this;
                s.ShowDialog();
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnSettings_Click", ex, true);
            }
        }
        private void btnStats_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (btnStats.IsChecked)
                {
                    CombatDetails currentFight = null;
                    if (Log != null && Log.LastFightIndex > 0)
                        currentFight = Log.Fights[Log.LastFightIndex];
                    StatPopOut.ShowActivated = false;
                    StatPopOut.SetStats(currentFight);
                    StatPopOut.Show();
                    StatPopOut.Owner = PopUpHost;
                    StatPopOut.Topmost = true;
                }
                else
                    StatPopOut.Hide();
            }
            catch (Exception ex)
            {
                HandleError("Error Toggling Stat Overview", "Error on btnStats_Click", ex, true);
            }
        }
        private void btnRaidDps_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (btnRaidDps.IsChecked)
                {
                    CombatDetails currentFight = null;
                    if (Log != null && Log.LastFightIndex > 0)
                        currentFight = Log.Fights[Log.LastFightIndex];
                    if (currentFight != null)
                    {
                        BindRaidDetails(currentFight);
                    }
                    else
                        RaidDpsPopOut.BindData(null);
                    RaidDpsPopOut.ShowActivated = false;
                    RaidDpsPopOut.Show();
                    RaidDpsPopOut.Owner = PopUpHost;
                    RaidDpsPopOut.Topmost = true;
                }
                else
                    RaidDpsPopOut.Hide();
            }
            catch (Exception ex)
            {
                HandleError("Error Toggling Raid DPS", "Error on btnRaidDps_Click", ex, true);
            }
        }
        public static SolidColorBrush WhiteBackground = new SolidColorBrush(System.Windows.Media.Colors.White);
        public static SolidColorBrush BlackBackground = new SolidColorBrush(System.Windows.Media.Colors.Black);
        public static SolidColorBrush TransparentBackground = new SolidColorBrush(System.Windows.Media.Colors.Transparent);
        public static void SetStreamMode(Window w, bool isStream, SolidColorBrush background)
        {
            if (w != null)
            {
                if (isStream)
                {
                    w.AllowsTransparency = false;
                     //w.WindowStyle = System.Windows.WindowStyle.SingleBorderWindow;
                    w.Background = background;
                }
                else
                {
                    w.AllowsTransparency = true;
                    //w.WindowStyle = System.Windows.WindowStyle.None;
                    w.Background = background;
                }
            }
        }
        private void FightTypeChangedHandler(object sender, RoutedEventArgs e)
        {
            if (Log != null && Log.LastFightIndex > 0) //Fight found
            {
                CombatDetails currentFight = Log.Fights[Log.LastFightIndex];
                UpdateStats(currentFight, Log.LogStatistics);
            }
        }
        //private TestStreamWindow TestStreamWin = new TestStreamWindow();
        //private void btnTestWindow_Click(object sender, RoutedEventArgs e)
        //{
        //    try
        //    {
        //        if (!TestStreamWin.IsVisible)
        //        {
        //            //TestStreamWin.ShowActivated = false;
        //            TestStreamWin.Show();
        //            //TestStreamWin.Owner = PopUpHost;
        //            //TestStreamWin.Topmost = true;
        //        }
        //        else
        //        {
        //            TestStreamWin.Hide();
        //        }

        //    }
        //    catch (Exception ex)
        //    {
        //        HandleError("Error", "Error on btnTestWindow_Click", ex);
        //    }
        //}
        private void btnRaidHps_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (btnRaidHps.IsChecked)
                {
                    CombatDetails currentFight = null;
                    if (Log != null && Log.LastFightIndex > 0)
                        currentFight = Log.Fights[Log.LastFightIndex];
                    if (currentFight != null)
                    {
                        BindRaidDetails(currentFight);
                    }
                    else
                        RaidHpsPopOut.BindData(null);
                    RaidHpsPopOut.ShowActivated = false;
                    RaidHpsPopOut.Show();
                    RaidHpsPopOut.Owner = PopUpHost;
                    RaidHpsPopOut.Topmost = true;
                }
                else
                    RaidHpsPopOut.Hide();
            }
            catch (Exception ex)
            {
                HandleError("Error Toggling Raid HPS", "Error on btnRaidHps_Click", ex, true);
            }
        }
        private void btnRaidTps_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (btnRaidTps.IsChecked)
                {
                    CombatDetails currentFight = null;
                    if (Log != null && Log.LastFightIndex > 0)
                        currentFight = Log.Fights[Log.LastFightIndex];
                    if (currentFight != null)
                    {
                        BindRaidDetails(currentFight);
                    }
                    else
                        RaidTpsPopOut.BindData(null);
                    RaidTpsPopOut.ShowActivated = false;
                    RaidTpsPopOut.Show();
                    RaidTpsPopOut.Owner = PopUpHost;
                    RaidTpsPopOut.Topmost = true;
                    //menuPopOuts.Focus();
                    //menuPopOuts.IsSubmenuOpen = true;
                }
                else
                    RaidTpsPopOut.Hide();
            }
            catch (Exception ex)
            {
                HandleError("Error Toggling Raid Threat", "Error on btnRaidTps_Click", ex, true);
            }
        }
        private void btnStatStopWatch_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (btnStatStopWatch.IsChecked)
                {
                    CombatDetails currentFight = null;
                    if (Log != null && Log.LastFightIndex > 0)
                        currentFight = Log.Fights[Log.LastFightIndex];
                    PvpStatsPopOut.ShowActivated = false;
                    PvpStatsPopOut.Show();
                    PvpStatsPopOut.Owner = PopUpHost;
                    PvpStatsPopOut.Topmost = true;
                }
                else
                    PvpStatsPopOut.Hide();
            }
            catch (Exception ex)
            {
                HandleError("Error Toggling Stopwatch", "Error on btnRaidTps_Click", ex, true);
            }
        }
        private void DragWindow(object sender, MouseButtonEventArgs args)
        {
            try
            {
                this.WindowState = WindowState.Normal;
                DragMove();
            }
            catch
            {
                //ignore
            }
        }
        private void Window_Deactivated(object sender, EventArgs e)
        {
            try
            {
                MovePopOutsToFront();
            }
            catch (Exception ex)
            {
                HandleError(null, "Error on Window_Deactivated", ex, true);
            }
        }
        private void btnMinimize_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                WindowState = WindowState.Minimized;
            }
            catch (Exception ex)
            {
                HandleError(null, "Error on btnMinimize_Click", ex, true);
            }
        }
        private void Window_Closed(object sender, EventArgs e)
        {
            try
            {
                SaveSettings();
                StopWatching();
                IsRaidPolling = false;
                StatPopOut.Close();
                PvpStatsPopOut.Close();
                RaidDpsPopOut.Close();
                RaidHpsPopOut.Close();
                RaidTpsPopOut.Close();
            }
            catch (Exception ex)
            {
                HandleError(null, "Error on Window_Closed", ex, true);
            }
        }
        private void MyTabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                int fightIndex = Convert.ToInt32(ddlFights.SelectedValue);
                if (fightIndex > 0)
                {
                    BindFightTabs(Log.Fights[fightIndex]);
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Changing Tabs", "Error on MyTabControl_SelectionChanged", ex, true);
            }
        }
        private void btnConnect_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(CurrentRaidGroup.Name))
                {
                    HandleError("You must select a Raid Group to join", null, null, true);
                }
                else
                {
                    if (!IsWatching)
                        btnWatch_Click(this, e);
                    CombatDetails c = new CombatDetails();
                    bool isInCombat = false;
                    if (Log.LastFightIndex > 0)
                    {
                        c = Log.Fights[Log.LastFightIndex];
                        isInCombat = Log.IsInCombat;
                    }
                    int requestedPollRate = 0;
                    //if (MyRandom.Next(0, 2) != 0)
                    //    RaidService = new ParsecRaidServiceSoapClient("ParsecRaidServiceSoap", AlternateServiceUrl);
                    //else
                    //    RaidService = new ParsecRaidServiceSoapClient();
                    SendRaidServerHeartBeat(c, isInCombat, ref requestedPollRate);
                    if (!isInCombat)
                    {
                        ConsecutiveOutOfCombatRaidSyncs = MaxOutOfCombatRaidSyncs;
                        LastCombatRaidSync = DateTime.UtcNow;
                    }
                    txtRaidGroup.Text = CurrentRaidGroup.Name.Capitalize();
                    btnConnect.Visibility = System.Windows.Visibility.Collapsed;
                    btnDisconnect.Visibility = System.Windows.Visibility.Visible;
                    SaveSettings();
                    //go to raid tab
                    MyTabControl.SelectedIndex = 6;
                    if (isInCombat)
                        BindRaidDetails(c);
                    BindConnectedRaiders(!(IsLastFightSelected));
                    IsRaidPolling = true;
                    btnRefreshRaid.IsEnabled = true;
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Connecting to Raid Service", "Error on btnConnect_Click", ex, true);
            }
        }
        private void btnDisconnect_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                DisconnectFromRaid(Log.CharacterName);
                SetDisconnected();
            }
            catch (Exception ex)
            {
                HandleError("Error Disconnecting", "Error on btnDisconnect_Click", ex, true);
            }
        }
        private void ddlRaidReportType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                int selectedFightIndex = Convert.ToInt32(ddlFights.SelectedValue);
                if (selectedFightIndex > 0)
                {
                    CombatDetails c = Log.Fights[selectedFightIndex];
                    BindRaidDetails(c);
                }
                SaveSettings();
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on ddlRaidReportType_SelectionChanged", ex, true);
            }
        }
        private void btnRaidGroup_Clicked(object sender, RoutedEventArgs e)
        {
            MenuItem m = ((MenuItem)sender);
            if (!m.IsChecked)
            {
                object o = m.Header;
                if (o != null)
                {
                    m.IsChecked = true;
                    foreach (object oi in menuRaidGroups.Items)
                    {
                        if (oi is MenuItem)
                        {
                            MenuItem i = (MenuItem)oi;
                            if (((string)i.Header) != "Manage")
                            {
                                if (i != m)
                                {
                                    i.IsChecked = false;
                                }
                            }
                        }
                    }
                    string groupName = (string)o;
                    RaidGroup c = RaidGroups.FirstOrDefault(x => string.Compare(x.Name, groupName, true) == 0);
                    if (c != null)
                    {
                        bool wasRaiding = IsRaidPolling;
                        if (IsRaidPolling)
                        {
                            //if you are raiding then stop
                            DisconnectFromRaid(Log.CharacterName);
                            SetDisconnected();
                        }
                        CurrentRaidGroup = c;
                        if (wasRaiding)
                        {
                            //if you were raiding reconnect
                            btnConnect_Click(btnConnect, new RoutedEventArgs());
                        }
                    }
                }
            }
        }
        private void btnManageGroups_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                SaveSettings();
                ManageRaidGroups s = new ManageRaidGroups();
                s.Owner = this;
                s.ShowDialog();
                BindRaidGroups();
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnManageGroups_Click", ex, true);
            }
        }
        private void btnHelp_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                AboutWindow s = new AboutWindow();
                s.Owner = this;
                s.ShowDialog();
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnHelp_Click", ex, true);
            }
        }
        private void btnRefreshRaid_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (IsRaidPolling)
                {
                    CombatDetails currentFight = new CombatDetails();
                    if (Log.LastFightIndex > 0)
                    {
                        currentFight = Log.Fights[Log.LastFightIndex];
                    }
                    int requestedPollRate = 0;
                    SendRaidServerHeartBeat(currentFight, false, ref requestedPollRate);
                    if (Log.LastFightIndex > 0)
                    {
                        if (IsLastFightSelected)
                        {
                            BindRaidDetails(currentFight);
                        }
                    }
                    BindConnectedRaiders(!IsLastFightSelected);
                    ConsecutiveRaidSyncErrors = 0;
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnHelp_Click", ex, true);
            }
        }
        private void OpenLogFileInOS(object sender, System.Windows.Navigation.RequestNavigateEventArgs e)
        {
            Process.Start(new ProcessStartInfo(e.Uri.AbsoluteUri));
            e.Handled = true;
        }

        #endregion

        #region Helper Methods
        private void SetDisconnected()
        {
            IsRaidPolling = false;
            btnRefreshRaid.IsEnabled = false;
            btnConnect.Visibility = System.Windows.Visibility.Visible;
            btnDisconnect.Visibility = System.Windows.Visibility.Collapsed;
        }
        private void MovePopOutsToFront()
        {
            if (this.btnStatStopWatch.IsChecked)
            {
                this.PvpStatsPopOut.Topmost = true;
                //this.StatPopOut.Activate();
                MoveToFront.MoveWindowToFront(PvpStatsPopOut.Title);
            }
            if (this.btnStats.IsChecked)
            {
                this.StatPopOut.Topmost = true;
                //this.StatPopOut.Activate();
                MoveToFront.MoveWindowToFront(StatPopOut.Title);
            }
            if (this.btnRaidDps.IsChecked)
            {
                this.RaidDpsPopOut.Topmost = true;
                //this.RaidDpsPopOut.Activate();
                MoveToFront.MoveWindowToFront(RaidDpsPopOut.Title);
            }
            if (this.btnRaidHps.IsChecked)
            {
                this.RaidHpsPopOut.Topmost = true;
                //this.RaidDpsPopOut.Activate();
                MoveToFront.MoveWindowToFront(RaidHpsPopOut.Title);
            }
            if (this.btnRaidTps.IsChecked)
            {
                this.RaidTpsPopOut.Topmost = true;
                //this.RaidDpsPopOut.Activate();
                MoveToFront.MoveWindowToFront(RaidTpsPopOut.Title);
            }
        }
        private void ProcessLogFile(bool isArchive)
        {
            //DateTime perfDate = DateTime.UtcNow;
            InitializeLogReader(LogFileFullPath, LogFolderPath, isArchive);
            //Debug.WriteLine("InitLogReader: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
            //perfDate = DateTime.UtcNow;
            LogParser = new Parser();
            if (Log != null)
                PreviousLoggedCharacter = Log.CharacterName;
            Log = new CombatLog(LogParser, System.IO.Path.GetFileName(LogReader.FileName));
            //Debug.WriteLine("InitCombatLog: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
            //perfDate = DateTime.UtcNow;
            if (isArchive)
            {
                brdArchive.Visibility = System.Windows.Visibility.Visible;
                txtArchiveDate.Text = Parser.ParseDateFromFileName(Log.FileName, true).ToString("dddd, MMM. d h:mm tt");
                txtArchiveName.Text = Log.FileName;
                lnkArchiveName.NavigateUri = new Uri(LogFileFullPath);
            }
            else
            {
                brdArchive.Visibility = System.Windows.Visibility.Collapsed;
            }
            List<string> existingLines = LogReader.ReadNewContent();
            //Debug.WriteLine("ReadLog: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
            //perfDate = DateTime.UtcNow;
            Log.AddLines(existingLines);
            //Debug.WriteLine("ParseLog: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
            //Debug.WriteLine("LinesParsed: " + existingLines.Count.ToString());
             BindFightList(false, null, true);
            UpdateCharacterName();
            PvpStatsPopOut.StopIfWatching();
            PvpStatsPopOut.ClearStats();
            BindRaidDetails(null);
            BindConnectedRaiders(true);

            //perfDate = DateTime.UtcNow;
            //Debug.WriteLine("PreBoundData: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
            if (Log.LastFightIndex > 0) //Fight found
            {
                CombatDetails currentFight = Log.Fights[Log.LastFightIndex];
                //Debug.WriteLine("FightEventCount: " + currentFight.Events.Count.ToString());
                if (currentFight.Events.Count > 0)
                {
                    SetFightDropDownValue(Log.LastFightIndex);
                    UpdateStats(currentFight, Log.LogStatistics);
                    BindFightTabs(currentFight);
                    BindCombatLog(currentFight.Events, true);
                    BindRaidDetails(currentFight);
                }
            }
            else
            {
                //clear interface
                UpdateStats(null, null);
                ClearFightTabs();
                BindCombatLog(new List<CombatLogEvent>(), true);
            }
            //perfDate = DateTime.UtcNow;
            //Debug.WriteLine("BoundData: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
        }
        private void UpdateCharacterName()
        {
            string characterName = "Character";
            if (Log != null && !string.IsNullOrEmpty(Log.CharacterName))
                characterName = Log.CharacterName;
            lblCharacterName.Text = characterName;
        }
        private void UpdateStats(CombatDetails fightDetails, CombatStatistics logStats)
        {

            if ((rdoThisFight.IsChecked ?? false) && fightDetails != null && fightDetails.CombatTimeSpan.TotalSeconds > 0)
            {
                lblLengthValue.Text = String.Format("{0:hh\\:mm\\:ss\\.fff}", fightDetails.CombatTimeSpan);
                lblApmValue.Text = (fightDetails.Activates / fightDetails.CombatTimeSpan.TotalMinutes).ToString("#0.0");
                lblDamageOutValue.Text = fightDetails.DamageOut.ToString("#,###,###,##0");
                lblThreatValue.Text = fightDetails.Threat.ToString("#,###,###,##0");
                lblHealsOutValue.Text = fightDetails.HealOut.ToString("#,###,###,##0");
                lblDamageInValue.Text = fightDetails.DamageIn.ToString("#,###,###,##0");
                lblHealsInValue.Text = fightDetails.HealIn.ToString("#,###,###,##0");
                lblDamageOutValue2.Text = (fightDetails.DamageOut / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblThreatValue2.Text = (fightDetails.Threat / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsOutValue2.Text = (fightDetails.HealOut / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblDamageInValue2.Text = (fightDetails.DamageIn / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsInValue2.Text = (fightDetails.HealIn / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsOutValue3.Text = (fightDetails.HealOut > 0 ? ((double)fightDetails.EffectiveHealOut / (double)fightDetails.HealOut) : 0).ToString("##0.0%");
                lblHealsOutValue4.Text = (fightDetails.EffectiveHealOut / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");

            }
            else if ((rdoAllFights.IsChecked ?? false) && logStats != null && logStats.CombatTimeSpan.TotalSeconds > 0)
            {
                lblLengthValue.Text = String.Format("{0:hh\\:mm\\:ss\\.fff}", logStats.CombatTimeSpan);
                lblApmValue.Text = (logStats.Activates / logStats.CombatTimeSpan.TotalMinutes).ToString("#0.0");
                lblDamageOutValue.Text = logStats.DamageOut.ToString("#,###,###,##0");
                lblThreatValue.Text = logStats.Threat.ToString("#,###,###,##0");
                lblHealsOutValue.Text = logStats.HealOut.ToString("#,###,###,##0");
                lblDamageInValue.Text = logStats.DamageIn.ToString("#,###,###,##0");
                lblHealsInValue.Text = logStats.HealIn.ToString("#,###,###,##0");
                lblDamageOutValue2.Text = (logStats.DamageOut / logStats.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblThreatValue2.Text = (logStats.Threat / logStats.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsOutValue2.Text = (logStats.HealOut / logStats.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblDamageInValue2.Text = (logStats.DamageIn / logStats.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsInValue2.Text = (logStats.HealIn / logStats.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsOutValue3.Text = (logStats.HealOut > 0 ? ((double)logStats.EffectiveHealOut / (double)logStats.HealOut) : 0).ToString("##0.0%");
                lblHealsOutValue4.Text = (logStats.EffectiveHealOut / logStats.CombatTimeSpan.TotalSeconds).ToString("#,##0");
            }
            else
            {
                lblLengthValue.Text = "";
                lblApmValue.Text = "";
                lblDamageOutValue.Text = "";
                lblThreatValue.Text = "";
                lblHealsOutValue.Text = "";
                lblDamageInValue.Text = "";
                lblHealsInValue.Text = "";
                lblDamageOutValue2.Text = "";
                lblThreatValue2.Text = "";
                lblHealsOutValue2.Text = "";
                lblDamageInValue2.Text = "";
                lblHealsInValue2.Text = "";
                lblHealsOutValue3.Text = "";
                lblHealsOutValue4.Text = "";
            }

            if (fightDetails != null && fightDetails.CombatTimeSpan.TotalSeconds > 0 && fightDetails.CompanionInFight)
            {
                grdCompanion.Visibility = !fightDetails.CompanionInFight ? System.Windows.Visibility.Collapsed : System.Windows.Visibility.Visible;
                txtCompanionName.Text = fightDetails.Companion != null ? fightDetails.Companion.DisplayName : "Companion";
                lblCompDamageOutValue.Text = fightDetails.CompanionDamageOut.ToString("#,###,###,##0");
                lblCompThreatValue.Text = fightDetails.CompanionThreat.ToString("#,###,###,##0");
                lblCompHealsOutValue.Text = fightDetails.CompanionHealOut.ToString("#,###,###,##0");
                lblCompDamageInValue.Text = fightDetails.CompanionDamageIn.ToString("#,###,###,##0");
                lblCompHealsInValue.Text = fightDetails.CompanionHealIn.ToString("#,###,###,##0");
                lblCompDamageOutValue2.Text = (fightDetails.CompanionDamageOut / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblCompThreatValue2.Text = (fightDetails.CompanionThreat / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblCompHealsOutValue2.Text = (fightDetails.CompanionHealOut / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblCompDamageInValue2.Text = (fightDetails.CompanionDamageIn / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblCompHealsInValue2.Text = (fightDetails.CompanionHealIn / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblCompHealsOutValue3.Text = (fightDetails.CompanionHealOut > 0 ? ((double)fightDetails.CompanionEffectiveHealOut / (double)fightDetails.CompanionHealOut) : 0).ToString("##0.0%");
                lblCompHealsOutValue4.Text = (fightDetails.CompanionEffectiveHealOut / fightDetails.CombatTimeSpan.TotalSeconds).ToString("#,##0");
            }
            else
            {
                grdCompanion.Visibility = System.Windows.Visibility.Collapsed;
                txtCompanionName.Text = "Companion";
                lblCompDamageOutValue.Text = "";
                lblCompThreatValue.Text = "";
                lblCompHealsOutValue.Text = "";
                lblCompDamageInValue.Text = "";
                lblCompHealsInValue.Text = "";
                lblCompDamageOutValue2.Text = "";
                lblCompThreatValue2.Text = "";
                lblCompHealsOutValue2.Text = "";
                lblCompDamageInValue2.Text = "";
                lblCompHealsInValue2.Text = "";
                lblCompHealsOutValue3.Text = "";
                lblCompHealsOutValue4.Text = "";
            }
        }
        private void BindFightTabs(CombatDetails currentFight)
        {
            //DateTime perfDate = DateTime.UtcNow;
            switch (MyTabControl.SelectedIndex)
            {
                case 1://DamageOut
                    {
                        List<AbilityDisplayItem> damageOutDetails = currentFight.Events.ApplyFilter(new CombatLogFilterParameters() { ShowOutgoingDamage = true })
                                .GroupBy(x => x.Ability.DisplayName)
                                .Select(x => new AbilityDisplayItem()
                                {
                                    Name = x.Key,
                                    UseCount = x.Count(),
                                    CritCount = x.Count(y => y.IsCrit),
                                    Total = x.Sum(y => y.Value),
                                    CritTotal = x.Sum(y => y.IsCrit ? y.Value : 0),
                                    DamageTypes = x.Where(z => z.DamageType != null && z.DamageType.EntityId != 0).GroupBy(z => z.DamageType.EntityId).Select(z => new DamageTypeDisplay() { DamageType = z.First().DamageType, DamageSum = z.Sum(v => v.Value) }).ToList()
                                }).OrderByDescending(x => x.Total).ToList();
                        double sumTotal = currentFight.DamageOut;
                        damageOutDetails.ForEach(x =>
                        {
                            x.Pct = sumTotal > 0 ? ((double)x.Total / sumTotal) * 100 : 0;
                            x.CritPct = ((double)x.CritCount / (double)x.UseCount) * 100;
                            x.Avg = (double)x.Total / (double)x.UseCount;
                            x.CritAvg = x.CritCount > 0 ? (double)x.CritTotal / (double)x.CritCount : 0;
                            x.AvgNonCrit = (x.UseCount - x.CritCount) > 0 ? (double)(x.Total - x.CritTotal) / (double)(x.UseCount - x.CritCount) : 0;
                            if (x.DamageTypes.Count == 1)
                            {
                                var dt =(x.DamageTypes.First());
                                x.DamageColor = dt.DamageType.GetDamageColor();
                                x.DamageTypeName = dt.DamageType.DisplayName.Capitalize();
                            }
                            else if (x.DamageTypes.Count > 1)
                            {
                                x.DamageColor = DisplayHelpers.GrayTransparentBrush;
                                x.DamageTypeName = string.Join(", ", x.DamageTypes.Where(y => !string.IsNullOrEmpty(y.DamageType.DisplayName)).Select(y => y.DamageType.DisplayName.Capitalize() + ": " + y.DamageSum.ToString("#,###,##0")).Distinct().ToArray());
                            }
                        });
                        icDamageOutDetails.ItemsSource = damageOutDetails;
                    }
                    break;
                case 2://Heal Out
                    {
                        var healsOut = currentFight.Events.ApplyFilter(new CombatLogFilterParameters() { ShowOutgoingHeals = true });
                        List<AbilityDisplayItem> healOutDetails = healsOut.GroupBy(x => x.Ability.DisplayName)
                            .Select(x => new AbilityDisplayItem()
                            {
                                Name = x.Key,
                                UseCount = x.Count(),
                                CritCount = x.Count(y => y.IsCrit),
                                Total = x.Sum(y => y.Value),
                                CritTotal = x.Sum(y => y.IsCrit ? y.Value : 0),
                                EffectiveTotal = x.Sum(y => y.EffectiveHeal)
                            }).OrderByDescending(x => x.Total).ToList();

                        int sumTotal = currentFight.HealOut;
                        healOutDetails.ForEach(x =>
                        {
                            x.Pct = sumTotal > 0 ? ((double)x.Total / sumTotal) * 100 : 0;
                            x.CritPct = ((double)x.CritCount / (double)x.UseCount) * 100;
                            x.Avg = (double)x.Total / (double)x.UseCount;
                            x.CritAvg = x.CritCount > 0 ? (double)x.CritTotal / (double)x.CritCount : 0;
                            x.AvgNonCrit = (x.UseCount - x.CritCount) > 0 ? (double)(x.Total - x.CritTotal) / (double)(x.UseCount - x.CritCount) : 0;
                            x.EffectivePct = x.Total > 0 ? ((double)(x.EffectiveTotal) / (double)(x.Total)) * 100 : 0;
                        });
                        icHealOutDetails.ItemsSource = healOutDetails;
                        int shieldsCasts = currentFight.Events.ApplyFilter(new CombatLogFilterParameters() { ShowAbilities = true }).Where(x => x.Source.IsThisPlayer && x.Ability.IsShieldAbility()).Count();
                        shieldsCasts += currentFight.Events.ApplyFilter(new CombatLogFilterParameters() { ShowCharacterEffects = true }).Where(x => x.EffectType.EntityId == CombatLogConstants.EffectTypes.ApplyEffect && x.EffectName.IsShieldEffect()).Count();
                        txtAbsorbCast.Text = shieldsCasts.ToString("###,###,##0") + " (" + (currentFight.CombatTimeSpan.TotalMinutes > 0 ? (double)shieldsCasts / currentFight.CombatTimeSpan.TotalMinutes : 0).ToString("#,###,##0.#") + ")";
                    }
                    break;
                case 3://Damage In
                    {
                        List<NestedAbilityDisplayItem> damageInDetails = currentFight.Events.ApplyFilter(new CombatLogFilterParameters() { ShowIncomingDamage = true })
                            .GroupBy(x => x.Source.DisplayNameWithOwner)
                            .Select(x => new NestedAbilityDisplayItem()
                            {
                                Name = x.Key,
                                UseCount = x.Count(),
                                ShieldCount = x.Count(y => y.Mitigation != null && y.Value > 0),
                                DefenseCount = x.Count(y => y.Mitigation != null && y.Value == 0),
                                Total = x.Sum(y => y.Value),
                                AbsorbedTotal = x.Sum(y => y.AbsorbedValue),
                                DpsIn = currentFight.CombatTimeSpan.TotalSeconds > 0 ? x.Sum(y => y.Value) / currentFight.CombatTimeSpan.TotalSeconds : 0d,
                                Abilities = x.GroupBy(y => y.Ability.DisplayName).Select(y => new AbilityDisplayItem()
                                {
                                    Name = y.Key,
                                    UseCount = y.Count(),
                                    ShieldCount = y.Count(z => z.Mitigation != null && z.Mitigation.EntityId == CombatLogConstants.Mitigation.Shield),
                                    DefenseCount = y.Count(z => z.Mitigation != null && z.Mitigation.EntityId != CombatLogConstants.Mitigation.Shield), //z.Mitigation.MitigationIsDefense()),
                                    AbsorbedTotal = y.Sum(z => z.AbsorbedValue),
                                    Total = y.Sum(z => z.Value),
                                    DamageTypes = y.Where(z => z.DamageType != null && z.DamageType.EntityId != 0).GroupBy(z => z.DamageType.EntityId).Select(z => new DamageTypeDisplay() { DamageType = z.First().DamageType, DamageSum = z.Sum(v => v.Value) }).ToList()
                                }).OrderByDescending(y => y.Total).ToList()
                            }).OrderByDescending(x => x.Total).ToList();
                        int sumTotal = currentFight.DamageIn;
                        int internalTotal = 0;
                        int elementalTotal = 0;
                        int kineticTotal = 0;
                        int energyTotal = 0;
                        int countTotal = 0;
                        int defenseCountTotal = 0;
                        int shieldCountTotal = 0;
                        int absorbTotal = currentFight.AbsorbedIn;
                        damageInDetails.ForEach(x =>
                        {
                            x.Pct = sumTotal > 0 ? ((double)x.Total / (double)sumTotal) * 100 : 0;
                            x.ShieldPct = ((double)x.ShieldCount / (double)x.UseCount) * 100;
                            x.DefensePct = ((double)x.DefenseCount / (double)x.UseCount) * 100;
                            x.Abilities.ForEach(y =>
                            {
                                y.Pct = x.Total > 0 ? ((double)y.Total / x.Total) * 100 : 0;
                                y.ShieldPct = ((double)y.ShieldCount / (double)y.UseCount) * 100;
                                y.DefensePct = ((double)y.DefenseCount / (double)y.UseCount) * 100;
                                y.Avg = (double)y.Total / (double)y.UseCount;
                                if (y.DamageTypes.Count == 1)
                                {
                                    var dt = (y.DamageTypes.First());
                                    y.DamageColor = dt.DamageType.GetDamageColor();
                                    y.DamageTypeName = dt.DamageType.DisplayName.Capitalize();
                                }
                                else if (y.DamageTypes.Count > 1)
                                {
                                    y.DamageColor = DisplayHelpers.GrayTransparentBrush;
                                    y.DamageTypeName = string.Join(", ", y.DamageTypes.Where(z => !string.IsNullOrEmpty(z.DamageType.DisplayName)).Select(z => z.DamageType.DisplayName.Capitalize() + ": " + z.DamageSum.ToString("#,###,##0")).Distinct().ToArray());
                                }
                                foreach (var dt in y.DamageTypes)
                                {
                                    if (dt.DamageType.DisplayName != null && x.DamageByType.ContainsKey(dt.DamageType.DisplayName))
                                        x.DamageByType[dt.DamageType.DisplayName] += dt.DamageSum;
                                    else if (y.DamageTypeName != null)
                                        x.DamageByType.Add(dt.DamageType.DisplayName, dt.DamageSum);
                                    switch (dt.DamageType.EntityId)
                                    {
                                        case CombatLogConstants.DamageTypes.Elemental:
                                            {
                                                elementalTotal += dt.DamageSum;
                                                break;
                                            }
                                        case CombatLogConstants.DamageTypes.Internal:
                                            {
                                                internalTotal += dt.DamageSum;
                                                break;
                                            }
                                        case CombatLogConstants.DamageTypes.Kinetic:
                                            {
                                                kineticTotal += dt.DamageSum;
                                                break;
                                            }
                                        case CombatLogConstants.DamageTypes.Energy:
                                            {
                                                energyTotal += dt.DamageSum;
                                                break;
                                            }
                                    }
                                }
                                countTotal += y.UseCount;
                                defenseCountTotal += y.DefenseCount;
                                shieldCountTotal += y.ShieldCount;
                            });
                        });
                        icDamageInDetails.ItemsSource = damageInDetails;
                        txtElementalDamageIn.Text = (sumTotal > 0 ? ((double)elementalTotal / (double)sumTotal) : 0.0).ToString("##0.0%");
                        txtInternalDamageIn.Text = (sumTotal > 0 ? ((double)internalTotal / (double)sumTotal) : 0.0).ToString("##0.0%");
                        txtEnergyDamageIn.Text = (sumTotal > 0 ? ((double)energyTotal / (double)sumTotal) : 0.0).ToString("##0.0%");
                        txtKineticDamageIn.Text = (sumTotal > 0 ? ((double)kineticTotal / (double)sumTotal) : 0.0).ToString("##0.0%");
                        txtDefenseIn.Text = (countTotal > 0 ? ((double)defenseCountTotal / (double)countTotal) : 0.0).ToString("##0.0%");
                        txtShieldIn.Text = (countTotal > 0 ? ((double)shieldCountTotal / (double)countTotal) : 0.0).ToString("##0.0%");
                        txtAbsorbIn.Text = (countTotal > 0 ? ((double)absorbTotal / (double)sumTotal) : 0.0).ToString("##0.0%");
                        txtAbsorbPerSec.Text = (currentFight.CombatTimeSpan.TotalSeconds > 0 ? (double)absorbTotal / currentFight.CombatTimeSpan.TotalSeconds : 0).ToString("#,###,##0");
                        txtElementalDamageIn.ToolTip = elementalTotal.ToString("###,###,##0");
                        txtInternalDamageIn.ToolTip = internalTotal.ToString("###,###,##0");
                        txtEnergyDamageIn.ToolTip = energyTotal.ToString("###,###,##0");
                        txtKineticDamageIn.ToolTip = kineticTotal.ToString("###,###,##0");
                        txtAbsorbIn.ToolTip = absorbTotal.ToString("###,###,##0");
                    }
                    break;
                case 4: //Heal In
                    {
                        List<NestedAbilityDisplayItem> healInDetails = currentFight.Events.ApplyFilter(new CombatLogFilterParameters() { ShowIncomingHeals = true })
                            .GroupBy(x => x.Source.DisplayNameWithOwner)
                            .Select(x => new NestedAbilityDisplayItem()
                            {
                                Name = x.Key,
                                UseCount = x.Count(),
                                Total = x.Sum(y => y.Value),
                                HpsIn = currentFight.CombatTimeSpan.TotalSeconds > 0 ? x.Sum(y => y.Value) / currentFight.CombatTimeSpan.TotalSeconds : 0d,
                                Abilities = x.GroupBy(y => y.Ability.DisplayName).Select(y => new AbilityDisplayItem()
                                {
                                    Name = y.Key,
                                    UseCount = y.Count(),
                                    Total = y.Sum(z => z.Value)
                                }).OrderByDescending(y => y.Total).ToList()
                            }).OrderByDescending(x => x.Total).ToList();
                        double sumTotal = currentFight.HealIn;
                        healInDetails.ForEach(x =>
                        {
                            x.Pct = sumTotal > 0 ? ((double)x.Total / sumTotal) * 100 : 0;
                            x.Abilities.ForEach(y =>
                            {
                                y.Pct = x.Total > 0 ? ((double)y.Total / x.Total) * 100 : 0;
                                y.Avg = (double)y.Total / (double)y.UseCount;
                            });
                        });
                        icHealInDetails.ItemsSource = healInDetails;
                    }
                    break;
                case 5: //threat
                    {
                        List<AbilityDisplayItem> threatList = (from t in currentFight.ThreatList
                                                              join p in currentFight.Participants on t.Key equals p.UniqueId
                                                               select new AbilityDisplayItem
                                                              {
                                                                  Name = p.DisplayNameWithOwner,
                                                                  Threat = t.Value,
                                                                  Tps = currentFight.CombatTimeSpan.TotalSeconds > 0 ? t.Value / currentFight.CombatTimeSpan.TotalSeconds : 0,
                                                                  Pct = 0.0
                                                              }).OrderByDescending(x => x.Threat).ToList();
                        double sumTotal = threatList.Sum(x => x.Threat);
                        threatList.ForEach(x =>
                        {
                            x.Pct = (x.Threat / sumTotal) * 100;
                        });
                        //threatList.Insert(0, new AbilityDisplayItem() { Name = "Heal Threat", Pct = 0, Threat = currentFight.HealThreat, Tps = currentFight.HealThreat / currentFight.CombatTimeSpan.TotalSeconds });
                        icThreatDetails.ItemsSource = threatList;
                        txtHealThreat.Text = currentFight.HealThreat.ToString("###,###,##0") + " (" + (currentFight.CombatTimeSpan.TotalSeconds > 0 ? (double)currentFight.HealThreat / currentFight.CombatTimeSpan.TotalSeconds : 0).ToString("#,###,##0") + ")";
                    }
                    break;
            }
            
            if (btnStats.IsChecked)
                StatPopOut.SetStats(currentFight);
        }
        private void ClearFightTabs()
        {
            DateTime perfDate = DateTime.UtcNow;
            icDamageOutDetails.ItemsSource = new List<AbilityDisplayItem>();
            icHealOutDetails.ItemsSource = new List<AbilityDisplayItem>();
            icDamageInDetails.ItemsSource = new List<NestedAbilityDisplayItem>();
            icHealInDetails.ItemsSource = new List<NestedAbilityDisplayItem>();
            icThreatDetails.ItemsSource = new List<AbilityDisplayItem>();
            if (btnStats.IsChecked)
                StatPopOut.SetStats(null);
            //Debug.WriteLine("ClearFightTabs: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
            txtElementalDamageIn.Text = "0%";
            txtInternalDamageIn.Text = "0%";
            txtEnergyDamageIn.Text = "0%";
            txtKineticDamageIn.Text = "0%";
            txtDefenseIn.Text = "0%";
            txtShieldIn.Text = "0%";
            txtElementalDamageIn.ToolTip = null;
            txtInternalDamageIn.ToolTip = null;
            txtEnergyDamageIn.ToolTip = null;
            txtKineticDamageIn.ToolTip = null;
            txtHealThreat.Text = "0 (0)";
            txtAbsorbCast.Text = "0 (0)";
        }
        private void BindCombatLog(List<CombatLogEvent> newEvents, bool clearPrevious)
        {
            DateTime perfDate = DateTime.UtcNow;
            if (clearPrevious && newEvents.Count == 0)
            {
                MyCombatLogControl.ClearContent();
            }
            else
            {
                IEnumerable<CombatLogEvent> filteredEvents = newEvents;
                filteredEvents = filteredEvents.ApplyFilter(LogFilterParameters);
                IEnumerable<string> filteredStrings = filteredEvents.Select(x => x.ToString(Log.CharacterName, true));
                filteredStrings = filteredStrings.ApplyFilter(LogFilterParameters);
                string[] events = filteredStrings.ToArray();
                if (clearPrevious)
                    MyCombatLogControl.SetLogContent(events);
                else
                    MyCombatLogControl.AppendLogContent(events);
            }
            //Debug.WriteLine("BindCombatLog: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
        }
        private void BindFightList(bool retainSelection, int? newSelectedValue, bool isArchive)
        {
            int? previousSelection = (int?)ddlFights.SelectedValue;
            DateTime perfDate = DateTime.UtcNow;
            var fights = Log.Fights.Select(x => new FightDisplayItem(x.Key, x.Value, isArchive)).ToList();
            ddlFights.ItemsSource = fights;
            //ddlFights.DisplayMemberPath = "Text";
            ddlFights.SelectedValuePath = "FightIndex";
            if (retainSelection && previousSelection.HasValue)
                SetFightDropDownValue(previousSelection.Value);
            else if (newSelectedValue.HasValue)
                SetFightDropDownValue(newSelectedValue.Value);

            //Debug.WriteLine("BindFightTabs: " + DateTime.UtcNow.Subtract(perfDate).TotalMilliseconds);
        }
        private void StartWatching()
        {
            IsWatching = true;
            ProcessLogFile(false);
            LogReader.StartWatchingFile();
            LogReader.StartWatchingFolder();
        }
        private void StopWatching()
        {
            IsWatching = false;
            if (LogReader != null)
            {
                LogReader.StopWatchingFile();
                LogReader.StopWatchingFolder();
                LogReader.StopReading();
            }
            //archive data
            if (IsArchiveEnabled && IsRaidPolling && Log != null && Log.LastFightIndex > 0)
            {
                CombatDetails archivedFight = Log.Fights[Log.LastFightIndex];
                if (archivedFight.RaidDetails.Count > 0)
                {
                    //archive raid
                    RaidData.ArchiveRaidData(Log.FileName, new RaidData() { FightIndex = Log.LastFightIndex, RaidDetails = archivedFight.RaidDetails });
                }
            }
        }
        private void SetFightDropDownValue(int fightIndex)
        {
            if (ddlFights.HasItems)
            {
                ddlFights.SelectionChanged -= new SelectionChangedEventHandler(ddlFights_SelectionChanged);
                ddlFights.SelectedValue = fightIndex;
                ddlFights.SelectionChanged += new SelectionChangedEventHandler(ddlFights_SelectionChanged);

                int lastPossibleFightIndex = ddlFights.Items.Cast<FightDisplayItem>().Select(x => x.FightIndex).Max();
                int currentSelectedFightIndex = ddlFights.SelectedValue != null ? Convert.ToInt32(ddlFights.SelectedValue) : 0;
                IsLastFightSelected = lastPossibleFightIndex == currentSelectedFightIndex;
            }
            else
                IsLastFightSelected = true;
        }
        private void SetRaidReportTypeDropDownValue(string selectedName)
        {
            if (ddlRaidReportType.HasItems)
            {
                ddlRaidReportType.SelectionChanged -= new SelectionChangedEventHandler(ddlRaidReportType_SelectionChanged);
                if (string.IsNullOrEmpty(selectedName))
                    this.ddlRaidReportType.SelectedValue = "cbiDamageDone";
                else if (!this.ddlRaidReportType.Items.Contains(selectedName))
                    this.ddlRaidReportType.SelectedValue = "cbiDamageDone";
                else
                {
                    try
                    {
                        this.ddlRaidReportType.SelectedValue = selectedName;
                    }
                    catch
                    {
                        this.ddlRaidReportType.SelectedValue = "cbiDamageDone";
                    }
                }
                ddlRaidReportType.SelectionChanged += new SelectionChangedEventHandler(ddlRaidReportType_SelectionChanged);
            }
        }
        private void InitializeLogReader(string fileName, string folderPath, bool forceClear)
        {
            if (LogReader != null && (LogReader.FolderPath != folderPath || LogReader.FileName != fileName || forceClear))
            {
                LogReader.Dispose();
                LogReader = null;
            }
            if (LogReader == null)
            {
                LogReader = new RealTimeLogReader(fileName, folderPath, LogUpdatedHandler, NewLogCreatedHandler, LogReaderError);
                LogReader.TickWhenLogIsNotUpdated = true;
                LogReader.PollMilliseconds = PollingRate;
            }
        }
        public void ResetSettings(bool resetGroups)
        {
            if (!resetGroups)
            {
                var rg = RaidGroups;
                Properties.Settings.Default.Reset();
                LoadSettings();
                RaidGroups = rg;
                SaveSettings();
            }
            else
            {
                Properties.Settings.Default.Reset();
                LoadSettings();
            }
            BindRaidGroups();
            BindRecentFiles();
        }
        public void SaveSettings()
        {
            Properties.Settings.Default.CombatLogFolder = this.LogFolderPath;
            Properties.Settings.Default.MainWindowHeight = this.Height;
            Properties.Settings.Default.MainWindowWidth = this.Width;
            Properties.Settings.Default.MainWindowTop = this.Top;
            Properties.Settings.Default.MainWindowLeft = this.Left;
            Properties.Settings.Default.StatPopOutWidth = StatPopOut.Width;
            Properties.Settings.Default.StatPopOutTop = StatPopOut.Top;
            Properties.Settings.Default.StatPopOutLeft = StatPopOut.Left;
            Properties.Settings.Default.StatPopOutOpacity = StatPopOut.GetBackgroundOpacity();
            Properties.Settings.Default.RaidDpsPopOutWidth = RaidDpsPopOut.Width;
            Properties.Settings.Default.RaidDpsPopOutTop = RaidDpsPopOut.Top;
            Properties.Settings.Default.RaidDpsPopOutLeft = RaidDpsPopOut.Left;
            Properties.Settings.Default.RaidDpsPopOutOpacity = RaidDpsPopOut.GetBackgroundOpacity();
            Properties.Settings.Default.RaidHpsPopOutWidth = RaidHpsPopOut.Width;
            Properties.Settings.Default.RaidHpsPopOutTop = RaidHpsPopOut.Top;
            Properties.Settings.Default.RaidHpsPopOutLeft = RaidHpsPopOut.Left;
            Properties.Settings.Default.RaidHpsPopOutOpacity = RaidHpsPopOut.GetBackgroundOpacity();
            Properties.Settings.Default.RaidTpsPopOutWidth = RaidTpsPopOut.Width;
            Properties.Settings.Default.RaidTpsPopOutTop = RaidTpsPopOut.Top;
            Properties.Settings.Default.RaidTpsPopOutLeft = RaidTpsPopOut.Left;
            Properties.Settings.Default.RaidTpsPopOutOpacity = RaidTpsPopOut.GetBackgroundOpacity();
            Properties.Settings.Default.PvpStatsPopOutWidth = PvpStatsPopOut.Width;
            Properties.Settings.Default.PvpStatsPopOutTop = PvpStatsPopOut.Top;
            Properties.Settings.Default.PvpStatsPopOutLeft = PvpStatsPopOut.Left;
            Properties.Settings.Default.PvpStatsPopOutOpacity = PvpStatsPopOut.GetBackgroundOpacity();
            Properties.Settings.Default.LastUsedRaidKey = CurrentRaidGroup.Name; //string.Join(",", new string[] { txtRaidName.Text, txtRaidPass.Text });
            Properties.Settings.Default.MainTopMost = this.Topmost;
            Properties.Settings.Default.IsStreamMode = this.IsStreamMode;
            Properties.Settings.Default.MainOpacity = this.Opacity;
            Properties.Settings.Default.RaidGroups = this.RaidGroups.Serialize();
            Properties.Settings.Default.RecentFiles = this.RecentFiles.Serialize();
            if (this.ddlRaidReportType.SelectedValue != null && this.ddlRaidReportType.SelectedValue is string)
                Properties.Settings.Default.RaidDetailsSortKey = (string)this.ddlRaidReportType.SelectedValue;
            Properties.Settings.Default.Save();
        }
        public void LoadSettings()
        {
            if (!string.IsNullOrEmpty(Properties.Settings.Default.CombatLogFolder))
                this.LogFolderPath = Properties.Settings.Default.CombatLogFolder;
            else
                this.LogFolderPath = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), DefaultLogPath);
            this.Height = Properties.Settings.Default.MainWindowHeight;
            this.Width = Properties.Settings.Default.MainWindowWidth;
            this.Top = Properties.Settings.Default.MainWindowTop;
            this.Left = Properties.Settings.Default.MainWindowLeft;
            this.IsStreamMode = Properties.Settings.Default.IsStreamMode;
            StatPopOut.Width = Properties.Settings.Default.StatPopOutWidth;
            StatPopOut.Top = Properties.Settings.Default.StatPopOutTop;
            StatPopOut.Left = Properties.Settings.Default.StatPopOutLeft;
            StatPopOut.SetBackgroundOpacity(Properties.Settings.Default.StatPopOutOpacity);
            RaidDpsPopOut.Width = Properties.Settings.Default.RaidDpsPopOutWidth;
            RaidDpsPopOut.Top = Properties.Settings.Default.RaidDpsPopOutTop;
            RaidDpsPopOut.Left = Properties.Settings.Default.RaidDpsPopOutLeft;
            RaidDpsPopOut.SetBackgroundOpacity(Properties.Settings.Default.RaidDpsPopOutOpacity);
            RaidHpsPopOut.Width = Properties.Settings.Default.RaidHpsPopOutWidth;
            RaidHpsPopOut.Top = Properties.Settings.Default.RaidHpsPopOutTop;
            RaidHpsPopOut.Left = Properties.Settings.Default.RaidHpsPopOutLeft;
            RaidHpsPopOut.SetBackgroundOpacity(Properties.Settings.Default.RaidHpsPopOutOpacity);
            RaidTpsPopOut.Width = Properties.Settings.Default.RaidTpsPopOutWidth;
            RaidTpsPopOut.Top = Properties.Settings.Default.RaidTpsPopOutTop;
            RaidTpsPopOut.Left = Properties.Settings.Default.RaidTpsPopOutLeft;
            RaidTpsPopOut.SetBackgroundOpacity(Properties.Settings.Default.RaidTpsPopOutOpacity);
            PvpStatsPopOut.Width = Properties.Settings.Default.PvpStatsPopOutWidth;
            PvpStatsPopOut.Top = Properties.Settings.Default.PvpStatsPopOutTop;
            PvpStatsPopOut.Left = Properties.Settings.Default.PvpStatsPopOutLeft;
            PvpStatsPopOut.SetBackgroundOpacity(Properties.Settings.Default.PvpStatsPopOutOpacity);
            SetRaidReportTypeDropDownValue(Properties.Settings.Default.RaidDetailsSortKey);
            this.Topmost = Properties.Settings.Default.MainTopMost;
            if (Properties.Settings.Default.MainOpacity < .1)
                Properties.Settings.Default.MainOpacity = .1;
            this.Opacity = Properties.Settings.Default.MainOpacity;
            bool saveNeeded = false;
            if (!string.IsNullOrEmpty(Properties.Settings.Default.RaidGroups))
            {
                try
                {
                    RaidGroups = Properties.Settings.Default.RaidGroups.Deserialize<List<RaidGroup>>(false);
                }
                catch
                {
                    //fix raid groups
                    Properties.Settings.Default.RaidGroups = this.RaidGroups.Serialize();
                    saveNeeded = true;
                }
            }
            else
            {
                RaidGroups = new List<RaidGroup>();
            }
            if (!string.IsNullOrEmpty(Properties.Settings.Default.LastUsedRaidKey))
            {
                RaidGroup c = RaidGroups.FirstOrDefault(x => string.Compare(x.Name, Properties.Settings.Default.LastUsedRaidKey, true) == 0);
                if (c != null)
                {
                    CurrentRaidGroup = c;
                }
                else
                {
                    Properties.Settings.Default.LastUsedRaidKey = null;
                    saveNeeded = true;
                }
            }
            else
            {
                CurrentRaidGroup = new RaidGroup();
            }
            if (!string.IsNullOrEmpty(Properties.Settings.Default.RecentFiles))
            {
                try
                {
                    RecentFiles = Properties.Settings.Default.RecentFiles.Deserialize<List<string>>(false);
                }
                catch
                {
                    //fix raid groups
                    Properties.Settings.Default.RecentFiles = this.RecentFiles.Serialize();
                    saveNeeded = true;
                }
            }
            if (saveNeeded)
                Properties.Settings.Default.Save();

        }
        public void BindRaidGroups()
        {
            menuRaidGroups.Items.Clear();
            var items = RaidGroups.Select(x => new { Name = x.Name, IsChecked = string.Compare(x.Name, CurrentRaidGroup.Name, true) == 0}).ToList();
            foreach (var i in items)
            {
                MenuItem m = new MenuItem() { IsChecked = i.IsChecked, Header = i.Name, StaysOpenOnClick=true };
                m.Click += new RoutedEventHandler(btnRaidGroup_Clicked);
                menuRaidGroups.Items.Add(m);
            }
            if (items.Count > 0)
                menuRaidGroups.Items.Add(new Separator());
            MenuItem manageButton = new MenuItem() { Header = "Manage" };
            manageButton.Click += new RoutedEventHandler(btnManageGroups_Click);
            menuRaidGroups.Items.Add(manageButton);
        }
        public void BindRecentFiles()
        {
            btnRecentLogs.Items.Clear();
            if (RecentFiles.Count == 0)
            {
                btnRecentLogs.IsEnabled = false;
            }
            else
            {
                btnRecentLogs.IsEnabled = true;
                var fs = RecentFiles.Reverse<string>();
                foreach (string f in fs)
                {
                    string name = System.IO.Path.GetFileName(f);
                    DateTime d = Parser.ParseDateFromFileName(name, true);
                    if (d != DateTime.Today)
                        name = d.ToString("dddd, MMM. d h:mm tt");
                    MenuItem m = new MenuItem() { Header = name, ToolTip=f};
                    m.Click += new RoutedEventHandler(btnRecentFile_Clicked);
                    btnRecentLogs.Items.Add(m);
                }
            }
        }
        private List<int> LastTenSyncs = new List<int>();
        private void SendRaidServerHeartBeat(CombatDetails c, bool syncData, ref int requestedPollRate)
        {
            if (c == null)
            {
                c = new CombatDetails();
            }
            if (string.IsNullOrEmpty(Log.CharacterName))
                throw new Exception("You must be logged in to a character to connect to a raid group");
            var p = new ParsecRaidParameters();
            p.RaidGroup = CurrentRaidGroup.Name;
            p.RaidPassword = CurrentRaidGroup.Password;
            p.Statistics = new ParsecRaidUser()
            {
                CharacterName = Log.CharacterName,
                RaidUserId = Log.RaidUserId,
                DamageOut = syncData ? c.DamageOut : 0,
                DamageIn = syncData ? c.DamageIn : 0,
                HealOut = syncData ? c.HealOut : 0,
                HealIn = syncData ? c.HealIn : 0,
                Threat = syncData ? c.Threat : 0,
                EffectiveHealOut = syncData ? c.EffectiveHealOut : 0,
                CombatTicks = syncData ? c.CombatTimeSpan.Ticks : 0,
                CombatStart = c.StartCombat.ToUniversalTime(),
                CombatEnd = c.EndCombat.HasValue ? c.EndCombat.Value.ToUniversalTime() : c.EndCombat,
                RaidEncounterId = syncData ? c.RaidEncounterId : 0
            };
            DateTime beforeDate = DateTime.UtcNow;
            var r = syncData ? RaidService.SyncRaidStats(p) : RaidService.GetRaidStats(p);
            TimeSpan syncTime = DateTime.UtcNow.Subtract(beforeDate);
            LastTenSyncs = LastTenSyncs.Take(9).ToList();
            LastTenSyncs.Add(Convert.ToInt32(syncTime.TotalMilliseconds));
            txtSyncMs.Text = "Ping " +  Convert.ToInt32(LastTenSyncs.Average()).ToString() + " ms";
            txtSyncMs.ToolTip = string.Join(Environment.NewLine, LastTenSyncs.Select(x => x + " ms").ToArray());
            if (syncData)
                LastCombatRaidSync = DateTime.UtcNow;
            LastRaidSyncDate = DateTime.UtcNow;
            if (string.IsNullOrEmpty(r.ErrorMessage))
            {
                requestedPollRate = r.MinimumPollingRate;
                if (r.LastUpdated.HasValue && r.LastUpdated == RaidLastUpdatedDate)
                {
                    ConsecutiveRaidHeartBeatWithNoUpdate++;
                }
                else
                {
                    ConsecutiveRaidHeartBeatWithNoUpdate = 0;
                    RaidLastUpdatedDate = r.LastUpdated;
                }
                //only update raid details if your in combat or you were in combat and raid connected
                if (syncData || c.DataHasBeenSynced)
                {
                    c.DataHasBeenSynced = true;
                    c.RaidDetails = r.Users.Where(x => x.LastCombatUpdate.HasValue).Select(x => new CombatStatistics()
                    {
                        CharacterName = x.CharacterName,
                        DamageIn = x.DamageIn,
                        DamageOut = x.DamageOut,
                        HealIn = x.HealIn,
                        HealOut = x.HealOut,
                        EffectiveHealOut = x.EffectiveHealOut,
                        Threat = x.Threat,
                        CombatTicks = x.CombatTicks,
                        CombatTimeSpan = new TimeSpan(x.CombatTicks),
                        StartCombat = x.CombatStart.HasValue ? x.CombatStart.Value : DateTime.MinValue,
                        EndCombat = x.CombatEnd,
                        RaidEncounterId = x.RaidEncounterId
                    }).ToList();
                }
                var u = r.Users.FirstOrDefault(x => x.CharacterName == Log.CharacterName);
                if (u != null)
                    Log.RaidUserId = u.RaidUserId;
                ConnectedRaiders = r.Users.Where(x => x.IsConnected).Select(x => x.CharacterName).OrderBy(x => x).ToList();
            }
            else
                throw new Exception(r.ErrorMessage);
        }
        private void DisconnectFromRaid(string characterName)
        {
            if (!string.IsNullOrEmpty(characterName))
            {
                var p = new ParsecRaidParameters();
                p.RaidGroup = CurrentRaidGroup.Name;
                p.RaidPassword = CurrentRaidGroup.Password;
                p.Statistics = new ParsecRaidUser()
                {
                    CharacterName = characterName
                };
                var r = RaidService.Disconnect(p);
                Log.RaidUserId = 0;
                this.ConnectedRaiders.Clear();
                if (!string.IsNullOrEmpty(r.ErrorMessage))
                    throw new Exception(r.ErrorMessage);
            }
            BindConnectedRaiders(true);
        }
        private void BindRaidDetails(CombatDetails currentFight)
        {
            //get selected fight
            if (currentFight == null)
            {
                icRaidDetails.ItemsSource = new List<RaidDisplayItem>();
                RaidDpsPopOut.BindData(null);
                RaidHpsPopOut.BindData(null);
                RaidTpsPopOut.BindData(null);
                return;
            }

            IEnumerable<RaidDisplayItem> displayItems = currentFight.RaidDetails.Select(x => new RaidDisplayItem()
            {
                Name = x.CharacterName,
                DamageOut = x.DamageOut,
                DamageIn = x.DamageIn,
                HealIn = x.HealIn,
                HealOut = x.HealOut,
                EffectiveHealOut = x.EffectiveHealOut,
                Threat = x.Threat,
                FightSeconds = x.CombatTimeSpan.TotalSeconds,
                DpsIn = x.CombatTimeSpan.TotalSeconds > 0 ? x.DamageIn / x.CombatTimeSpan.TotalSeconds : 0,
                DpsOut = x.CombatTimeSpan.TotalSeconds > 0 ? x.DamageOut / x.CombatTimeSpan.TotalSeconds : 0,
                HpsIn = x.CombatTimeSpan.TotalSeconds > 0 ? x.HealIn / x.CombatTimeSpan.TotalSeconds : 0,
                HpsOut = x.CombatTimeSpan.TotalSeconds > 0 ? x.HealOut / x.CombatTimeSpan.TotalSeconds : 0,
                EhpsOut = x.CombatTimeSpan.TotalSeconds > 0 ? x.EffectiveHealOut / x.CombatTimeSpan.TotalSeconds : 0,
                Tps = x.CombatTimeSpan.TotalSeconds > 0 ? x.Threat / x.CombatTimeSpan.TotalSeconds : 0,
                EffectivePct = x.HealOut > 0 ? ((double)x.EffectiveHealOut / (double)x.HealOut) * 100 : 0
            });

            if (cbiDamageDone.IsSelected)
                displayItems = displayItems.OrderByDescending(x => x.DamageOut);
            else if (cbiDamageIn.IsSelected)
                displayItems = displayItems.OrderByDescending(x => x.DamageIn);
            else if (cbiHealsDone.IsSelected)
                displayItems = displayItems.OrderByDescending(x => x.HealOut);
            else if (cbiEffectiveOut.IsSelected)
                displayItems = displayItems.OrderByDescending(x => x.EffectiveHealOut);
            else if (cbiThreat.IsSelected)
                displayItems = displayItems.OrderByDescending(x => x.Threat);
            List<RaidDisplayItem> raidDisplayItems = displayItems.ToList();
            icRaidDetails.ItemsSource = raidDisplayItems;
            if (btnRaidDps.IsChecked)
            {
                RaidDpsPopOut.BindData(raidDisplayItems);
            }
            if (btnRaidHps.IsChecked)
            {
                RaidHpsPopOut.BindData(raidDisplayItems);
            }
            if (btnRaidTps.IsChecked)
            {
                RaidTpsPopOut.BindData(raidDisplayItems);
            }
        }
        public void BindConnectedRaiders(bool clearRaiders)
        {
            if (!clearRaiders)
            {
                txtRaidGroupRaiders.Visibility = System.Windows.Visibility.Visible;
                txtRaidGroupRaiders.Text = string.Join(", ", ConnectedRaiders.ToArray());
            }
            else
            {
                txtRaidGroupRaiders.Visibility = System.Windows.Visibility.Collapsed;
                txtRaidGroupRaiders.Text = null;
            }
        }
        public void HandleError(string errorMessage, string logMessage, Exception ex, bool modalize)
        {
            try
            {
                if (!string.IsNullOrEmpty(errorMessage))
                {
                    if (ex != null)
                        errorMessage += Environment.NewLine + ex.Message;
                    if (MyMessageWindow != null)
                        MyMessageWindow.Close();
                    MyMessageWindow = new MessageWindow(errorMessage, true, false, false);
                    MyMessageWindow.ShowActivated = false;
                    MyMessageWindow.Owner = this;
                    if (modalize)
                        MyMessageWindow.ShowDialog();
                    else
                    {
                        this.IsEnabled = false;
                        MyMessageWindow.Show();
                    }
                }
                if (!string.IsNullOrEmpty(logMessage))
                {
                    if (ex == null)
                        Logger.WriteError(logMessage);
                    else
                        Logger.WriteError(logMessage, ex);
                }
            }
            catch (Exception ex2)
            {
                Logger.WriteError("Critical Error on Exception", ex2);
                MessageBox.Show(logMessage);
            }
        }
        private void AddRecentFile(string filePath)
        {
            RecentFiles.Remove(filePath);
            int skipcount = 0;
            if (RecentFiles.Count> 4)
                skipcount = RecentFiles.Count - 4;
            RecentFiles = RecentFiles.Skip(skipcount).Take(4).ToList();
            RecentFiles.Add(filePath);
        }
        //public void HandleMessage(string msg)
        //{
        //    if (!string.IsNullOrEmpty(msg))
        //    {
        //        if (MyMessageWindow != null)
        //            MyMessageWindow.Close();
        //        MyMessageWindow = new MessageWindow(msg, false, false, true);
        //        MyMessageWindow.Owner = this;
        //        MyMessageWindow.ShowDialog();
        //    }
        //}
        public void ShowNotes()
        {
            ReleaseNotes s = new ReleaseNotes();
            s.Owner = this;
            s.ShowDialog();
        }
        #endregion

    }
}
