﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ParsecParser;
using ParsecClient.Classes;
using ParsecClient.ParsecRaidWebService;

namespace ParsecClient
{
    /// <summary>
    /// Interaction logic for ManageRaidGroups.xaml
    /// </summary>
    public partial class ManageRaidGroups : Window
    {
        public ManageRaidGroups()
        {
            InitializeComponent();
        }

        private void btnDone_Click(object sender, RoutedEventArgs e)
        {

            try
            {
                this.Close();
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnDone_Click", ex);
                this.Close();
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

            MainWindow mw = ((MainWindow)Owner);
            lstGroups.ItemsSource = mw.RaidGroups.Select(x => new ListBoxItem() { Content = x.Name }).ToList();
        }

        private void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(txtRaidGroupName.Text) || string.IsNullOrEmpty(txtRaidGroupPass.Text))
                {
                    HandleError("You must provide a Raid Name and Password", null, null);
                }
                else if (txtRaidGroupName.Text.Length < 4 || txtRaidGroupPass.Text.Length < 4)
                {
                    HandleError("Raid Name and Password must be 4 characters or more", null, null);
                }
                else if (!txtRaidGroupName.Text.IsDigitsOrLettersOrSpaces())
                {
                    HandleError("Raid Name contain digits or letters or spaces and it cannot begin or end with a space", null, null);
                }
                else if (!txtRaidGroupPass.Text.IsDigitsOrLetters())
                {
                    HandleError("Raid Password must only contain digits or letters", null, null);
                }
                else
                {
                    ParsecRaidServiceSoapClient service = new ParsecRaidServiceSoapClient();
                    var keyVal = service.TestConnection(new ParsecRaidParameters() { RaidGroup = txtRaidGroupName.Text, RaidPassword = txtRaidGroupPass.Text });
                    if (string.IsNullOrEmpty(keyVal.ErrorMessage))
                    {
                        RaidGroup r = new RaidGroup() { Name = txtRaidGroupName.Text, Password = txtRaidGroupPass.Text };
                        MainWindow mw = ((MainWindow)Owner);
                        mw.RaidGroups.Add(r);
                        if (string.IsNullOrEmpty(mw.CurrentRaidGroup.Name))
                            mw.CurrentRaidGroup = r;
                        txtRaidGroupName.Text = null;
                        txtRaidGroupPass.Text = null;
                        lstGroups.ItemsSource = mw.RaidGroups.Select(x => new ListBoxItem() { Content = x.Name }).ToList();
                    }
                    else
                    {
                        HandleError("Error: " + keyVal.ErrorMessage, null, null);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnAdd_Click", ex);
            }
        }

        private void btnCreate_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(txtRaidGroupName.Text) || string.IsNullOrEmpty(txtRaidGroupPass.Text))
                {
                    HandleError("You must provide a Raid Name and Password", null, null);
                }
                else if (txtRaidGroupName.Text.Length < 4 || txtRaidGroupPass.Text.Length < 4)
                {
                    HandleError("Raid Name and Password must be 4 characters or more", null, null);
                }
                else if (!txtRaidGroupName.Text.IsDigitsOrLettersOrSpaces())
                {
                    HandleError("Raid Name contain digits or letters or spaces and it cannot begin or end with a space", null, null);
                }
                else if (!txtRaidGroupPass.Text.IsDigitsOrLetters())
                {
                    HandleError("Raid Password must only contain digits or letters", null, null);
                }
                else
                {
                    PasswordWindow pwin = new PasswordWindow("Enter an Admin Password that enables you to delete or modify this group later", CreateGroupAdminPassOk, null);
                    pwin.Owner = this;
                    pwin.ShowDialog();
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnCreate_Click", ex);
            }
        }

        private void btnRemoveGroup_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (!string.IsNullOrEmpty(txtCurrentName.Text))
                {
                    if (HandleMessage("Are you sure you want to remove this group from your list?", true))
                    {
                        MainWindow mw = ((MainWindow)Owner);
                        mw.RaidGroups.RemoveAll(x => string.Compare(x.Name, txtCurrentName.Text) == 0);
                        if (string.Compare(mw.CurrentRaidGroup.Name, txtCurrentName.Text) == 0)
                            mw.CurrentRaidGroup = new RaidGroup();
                        var items = mw.RaidGroups.Select(x => new ListBoxItem() { Content = x.Name }).ToList();
                        lstGroups.SelectedItem = null;
                        lstGroups.ItemsSource = items;
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnRemoveGroup_Click", ex);
            }
        }

        private void btnDeleteGroup_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (!string.IsNullOrEmpty(txtCurrentName.Text))
                {
                    if (HandleMessage("Are you sure you want to remove this group from the server?", true))
                    {
                        //if admin password is not set ask for it
                        if (string.IsNullOrEmpty(txtCurrentAdminPassword.Text))
                        {
                            PasswordWindow pwin = new PasswordWindow("Enter the Admin Password used when this group was created", DeletePasswordAdminEnteredOk, null);
                            pwin.Owner = this;
                            pwin.ShowDialog();
                        }
                        else
                        {
                            string m = DeleteRaidGroupFromServer(txtCurrentName.Text, txtCurrentPassword.Text, txtCurrentAdminPassword.Text);
                            if (string.IsNullOrEmpty(m))
                            {
                                MainWindow mw = ((MainWindow)Owner);
                                mw.RaidGroups.RemoveAll(x => string.Compare(x.Name, txtCurrentName.Text) == 0);
                                if (string.Compare(mw.CurrentRaidGroup.Name, txtCurrentName.Text) == 0)
                                    mw.CurrentRaidGroup = new RaidGroup();
                                lstGroups.ItemsSource = mw.RaidGroups.Select(x => new ListBoxItem() { Content = x.Name }).ToList();
                            }
                            else
                            {
                                HandleError(m, null, null);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on btnRemoveGroup_Click", ex);
            }
        }
        public void DeletePasswordAdminEnteredOk(object sender, PasswordOkEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(e.Password))
                {
                    HandleError("You must provide an Admin Password", null, null);
                }
                else
                {
                    PasswordWindow pwin = (PasswordWindow)sender;
                    string m = DeleteRaidGroupFromServer(txtCurrentName.Text, txtCurrentPassword.Text, e.Password);
                    if (string.IsNullOrEmpty(m))
                    {
                        MainWindow mw = ((MainWindow)Owner);
                        if (string.Compare(mw.CurrentRaidGroup.Name, txtCurrentName.Text) == 0)
                            mw.CurrentRaidGroup = new RaidGroup();
                        mw.RaidGroups.RemoveAll(x => string.Compare(x.Name, txtCurrentName.Text) == 0);
                        lstGroups.ItemsSource = mw.RaidGroups.Select(x => new ListBoxItem() { Content = x.Name }).ToList();
                        pwin.Close();
                    }
                    else
                    {
                        HandleError(m, null, null);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on EnterPassOk", ex);
            }
        }
        public void CreateGroupAdminPassOk(object sender, PasswordOkEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(e.Password))
                {
                    HandleError("You must provide an Admin Password", null, null);
                }
                else if (e.Password.Length < 4)
                {
                    HandleError("Admin Password must be 4 characters or more", null, null);
                }
                else if (!e.Password.IsDigitsOrLetters())
                {
                    HandleError("Admin Password must only contain digits or letters", null, null);
                }
                else
                {
                    PasswordWindow pwin = (PasswordWindow)sender;
                    ParsecRaidServiceSoapClient service = new ParsecRaidServiceSoapClient();
                    var keyVal = service.RequestRaidGroup(txtRaidGroupName.Text, txtRaidGroupPass.Text, e.Password);
                    if (keyVal.Success)
                    {
                        RaidGroup r = new RaidGroup() { Name = txtRaidGroupName.Text, Password = txtRaidGroupPass.Text, AdminPassword = e.Password };
                        MainWindow mw = ((MainWindow)Owner);
                        mw.RaidGroups.Add(r);
                        if (string.IsNullOrEmpty(mw.CurrentRaidGroup.Name))
                            mw.CurrentRaidGroup = r;
                        txtRaidGroupName.Text = null;
                        txtRaidGroupPass.Text = null;
                        lstGroups.ItemsSource = mw.RaidGroups.Select(x => new ListBoxItem() { Content = x.Name }).ToList();
                        pwin.Close();
                        HandleMessage("New Group Created", false);
                    }
                    else
                    {
                        pwin.Close();
                        HandleError("Group Rejected: " + keyVal.Message, null, null);
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on CreateGroupAdminPassOk", ex);
            }
        }
        private void HandleError(string errorMessage, string logMessage, Exception ex)
        {
            if (!string.IsNullOrEmpty(errorMessage))
            {
                if (ex != null)
                    errorMessage += Environment.NewLine + ex.Message;
                MessageWindow m = new MessageWindow(errorMessage, true, false, true);
                m.Owner = this;
                m.ShowDialog();
            }
            if (!string.IsNullOrEmpty(logMessage))
            {
                if (ex == null)
                    Logger.WriteError(logMessage);
                else
                    Logger.WriteError(logMessage, ex);
            }
        }
        public bool HandleMessage(string msg, bool showCancel)
        {
            if (!string.IsNullOrEmpty(msg))
            {
                MessageWindow m = new MessageWindow(msg, false, showCancel, true);
                m.Owner = this;
                return m.ShowDialog() ?? false;
            }
            return false;
        }

        private void lstGroups_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                txtCurrentName.Text = null;
                txtCurrentPassword.Text = null;
                txtCurrentAdminPassword.Text = null;
                if (lstGroups.SelectedItem != null)
                {
                    string selectedValue = (string)((ListBoxItem)lstGroups.SelectedItem).Content;
                    MainWindow mw = ((MainWindow)Owner);
                    RaidGroup r = mw.RaidGroups.FirstOrDefault(x => string.Compare(x.Name, selectedValue) == 0);
                    if (r != null)
                    {
                        txtCurrentName.Text = r.Name;
                        txtCurrentPassword.Text = r.Password;
                        txtCurrentAdminPassword.Text = r.AdminPassword;
                    }
                }

            }
            catch (Exception ex)
            {
                HandleError("Error", "Error on CreateGroupAdminPassOk", ex);
            }
        }
        private string DeleteRaidGroupFromServer(string name, string password, string adminPassword)
        {
            ParsecRaidServiceSoapClient service = new ParsecRaidServiceSoapClient();
            var r = service.TestConnection(new ParsecRaidParameters() { RaidGroup = name, RaidPassword = password });
            if (!string.IsNullOrEmpty(r.ErrorMessage))
            {
                return r.ErrorMessage;
            }
            else
            {
                var d = service.DeleteRaidGroup(name, adminPassword);
                return d.Message;
            }
        }
    }
}
