﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ParsecParser;
using ParsecClient.Classes;

namespace ParsecClient
{
    /// <summary>
    /// Interaction logic for StatsOverview.xaml
    /// </summary>
    public partial class PvpStatsOverview : Window
    {
        public MainWindow MainUIWindow;
        public DateTime? StopWatchStartTime = null;
        public CombatLog LocalLog = null;
        public PvpStatsOverview(MainWindow mainUIWindow)
        {
            InitializeComponent();
            MainUIWindow = mainUIWindow;
            LocalLog = new CombatLog();
            CommandBindings.Add(new CommandBinding(ApplicationCommands.Close, new ExecutedRoutedEventHandler(delegate(object sender, ExecutedRoutedEventArgs args) { this.Close(); })));
        }

        public void SetStats(List<CombatDetails> existingFights)
        {
            if (existingFights != null && existingFights.Count > 0)
            {
                if (StopWatchStartTime.HasValue)
                {
                    List<CombatLogEvent> events = existingFights.SelectMany(x => x.Events).Where(x => x.TimeStamp.CompareTo(StopWatchStartTime.Value) > 0).ToList();
                    SetStats(events);
                }
            }
            else
                ClearStats();
        }
        public void SetStats(List<CombatLogEvent> events)
        {
            Topmost = true;
            if (events != null && events.Count > 0)
            {
                if (StopWatchStartTime.HasValue)
                {
                    //List<CombatLogEvent> events = fights.SelectMany(x => x.Events).Where(x => x.TimeStamp.CompareTo(StopWatchStartTime.Value) > 0).ToList();
                    if (events.Count > 0)
                    {
                        foreach (var e in events)
                        {
                            LocalLog.AddEvent(e);
                        }
                        if (LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds > 0)
                        {
                            //string damageOut = LocalLog.LogStatistics.DamageOut.ToString("#,###,###,##0") + "  " + (LocalLog.LogStatistics.DamageOut / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                            //string damageIn = LocalLog.LogStatistics.DamageIn.ToString("#,###,###,##0") + "  " + (LocalLog.LogStatistics.DamageIn / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                            //string healOut = LocalLog.LogStatistics.HealOut.ToString("#,###,###,##0") + "  " + (LocalLog.LogStatistics.HealOut / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0"); // +"  " + (LocalLog.LogStatistics.HealOut > 0 ? ((double)LocalLog.LogStatistics.EffectiveHealOut / (double)LocalLog.LogStatistics.HealOut) : 0).ToString("##0.0%");
                            //string healIn = LocalLog.LogStatistics.HealIn.ToString("#,###,###,##0") + "  " + (LocalLog.LogStatistics.HealIn / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");

                            lblLengthValue.Text = String.Format("{0:hh\\:mm\\:ss\\.fff}", LocalLog.LogStatistics.CombatTimeSpan);
                            lblDamageOutValue.Text = LocalLog.LogStatistics.DamageOut.ToString("#,###,###,##0");
                            lblDamageOutValue2.Text = (LocalLog.LogStatistics.DamageOut / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                            lblHealsOutValue.Text = LocalLog.LogStatistics.HealOut.ToString("#,###,###,##0");
                            lblHealsOutValue2.Text = (LocalLog.LogStatistics.HealOut / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                            lblHealsOutValue3.Text = (LocalLog.LogStatistics.HealOut > 0 ? ((double)LocalLog.LogStatistics.EffectiveHealOut / (double)LocalLog.LogStatistics.HealOut) : 0).ToString("##0.0%");
                            lblHealsOutValue4.Text = (LocalLog.LogStatistics.EffectiveHealOut / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                            lblDamageInValue.Text = LocalLog.LogStatistics.DamageIn.ToString("#,###,###,##0");
                            lblDamageInValue2.Text = (LocalLog.LogStatistics.DamageIn / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                            lblHealsInValue.Text = LocalLog.LogStatistics.HealIn.ToString("#,###,###,##0");
                            lblHealsInValue2.Text = (LocalLog.LogStatistics.HealIn / LocalLog.LogStatistics.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                        }
                        else
                            ClearStats();
                    }
                    else
                        ClearStats();
                }
            }
            else
                ClearStats();
        }
        public void ClearStats()
        {
            lblLengthValue.Text = "";
            lblDamageOutValue.Text = "";
            lblHealsOutValue.Text = "";
            lblDamageInValue.Text = "";
            lblHealsInValue.Text = "";
            lblDamageOutValue2.Text = "";
            lblHealsOutValue2.Text = "";
            lblDamageInValue2.Text = "";
            lblHealsInValue2.Text = "";
            lblHealsOutValue3.Text = "";
            lblHealsOutValue4.Text = "";
        }
        public void StopIfWatching()
        {
            StopWatchStartTime = null;
            btnToggle.Content = "Start";
        }
        public double GetBackgroundOpacity()
        {
            return BorderBackground.Opacity;
        }
        public void SetBackgroundOpacity(double opacity)
        {

            BorderBackground.Opacity = opacity;
            BorderSolid.Opacity = opacity;
        }

        public void DragWindow(object sender, MouseButtonEventArgs args)
        {
            try
            {
                this.WindowState = WindowState.Normal;
                DragMove();
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out DragWindow", this.Title), ex, true);
            }
        }
        private void btnIncreaseOpacity_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (BorderBackground.Opacity < 1)
                {
                    BorderBackground.Opacity += .1;
                    BorderSolid.Opacity += .1;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseOpacity_Click", this.Title), ex, true);
            }
        }

        private void btnDecreaseOpacity_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (BorderBackground.Opacity > 0)
                {
                    BorderBackground.Opacity -= .1;
                    BorderSolid.Opacity -= .1;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnDecreaseOpacity_Click", this.Title), ex, true);
            }
        }

        private void btnIncreaseSize_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.Width < 1000)
                {
                    //this.Height += 10;
                    this.Width += 10;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }

        private void btnDecreaseSize_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.Width > 100)
                {
                    //this.Height -= 10;
                    this.Width -= 10;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (MainUIWindow != null)
                {
                    MainUIWindow.SaveSettings();
                    MainUIWindow.btnStatStopWatch.IsChecked = false;
                }
                this.Hide();
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }

        private void btnToggle_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                //set start time
                if (MainUIWindow != null)
                {
                    if (StopWatchStartTime.HasValue)
                    {
                        StopWatchStartTime = null;
                        btnToggle.Content = "Start";
                    }
                    else
                    {
                        LocalLog = new CombatLog();
                        if (MainUIWindow.Log != null)
                        {
                            StopWatchStartTime = MainUIWindow.Log.CurrentLogDate;
                            SetStats(MainUIWindow.Log.Fights.Values.ToList());
                        }
                        else
                            StopWatchStartTime = DateTime.Today;
                        btnToggle.Content = "Stop";
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnToggle_Click", this.Title), ex, true);
            }
        }
    }
}

