﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ParsecParser;
using ParsecClient.Classes;

namespace ParsecClient
{
    /// <summary>
    /// Interaction logic for RaidHps.xaml
    /// </summary>
    public partial class RaidTps : Window
    {
        public MainWindow MainUIWindow;
        public RaidTps(MainWindow mainUIWindow)
        {
            InitializeComponent();
            MainUIWindow = mainUIWindow;
            CommandBindings.Add(new CommandBinding(ApplicationCommands.Close, new ExecutedRoutedEventHandler(delegate(object sender, ExecutedRoutedEventArgs args) { this.Close(); })));
        }
        public void BindData(List<RaidDisplayItem> raidDetails)
        {
            Topmost = true;
            if (raidDetails != null && raidDetails.Count > 0)
            {
                icRaidTps.ItemsSource = raidDetails.Where(x => x.Threat > 0).OrderByDescending(x => x.Threat).Take(5).ToList();
            }
            else
            {
                icRaidTps.ItemsSource = new List<RaidDisplayItem>();
            }
        }
        public double GetBackgroundOpacity()
        {
            return BorderBackground.Opacity;
        }
        public void SetBackgroundOpacity(double opacity)
        {
            BorderBackground.Opacity = opacity;
            BorderSolid.Opacity = opacity;
        }

        public void DragWindow(object sender, MouseButtonEventArgs args)
        {
            try
            {
                this.WindowState = WindowState.Normal;
                DragMove();
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out DragWindow", this.Title), ex, true);
            }
        }
        private void btnIncreaseOpacity_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (BorderBackground.Opacity < 1)
                {
                    BorderBackground.Opacity += .1;
                    BorderSolid.Opacity += .1;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseOpacity_Click", this.Title), ex, true);
            }
        }

        private void btnDecreaseOpacity_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (BorderBackground.Opacity > 0)
                {
                    BorderBackground.Opacity -= .1;
                    BorderSolid.Opacity -= .1;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnDecreaseOpacity_Click", this.Title), ex, true);
            }
        }

        private void btnIncreaseSize_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.Width < 1000)
                {
                    //this.Height += 10;
                    this.Width += 10;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }

        private void btnDecreaseSize_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.Width > 100)
                {
                    //this.Height -= 10;
                    this.Width -= 10;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (MainUIWindow != null)
                {
                    MainUIWindow.SaveSettings();
                    MainUIWindow.btnRaidTps.IsChecked = false;
                }
                this.Hide();
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }
    }
}
