﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ParsecParser;
using ParsecClient.Classes;

namespace ParsecClient
{
    /// <summary>
    /// Interaction logic for Settings.xaml
    /// </summary>
    public partial class SettingsWindow : Window
    {
        public SettingsWindow()
        {
            InitializeComponent();
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            //undo changes
            try
            {
                ((MainWindow)Owner).LoadSettings();
                this.Close();
            }
            catch (Exception ex)
            {
                HandleError("Error Canceling Settings", "Error on btnCancel_Click", ex);
                this.Close();
            }
        }

        private void btnOk_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                //save changes
                if (!System.IO.Directory.Exists(txtLogLocation.Text))
                    HandleError("Folder does not exist", null, null);
                else
                {
                    MainWindow mw = ((MainWindow)Owner);
                    mw.LogFolderPath = txtLogLocation.Text;
                    mw.Topmost = chkAlwaysOnTop.IsChecked.HasValue ? chkAlwaysOnTop.IsChecked.Value : false;
                    mw.IsStreamMode = chkStreamMode.IsChecked.HasValue ? chkStreamMode.IsChecked.Value : false;
                    mw.SaveSettings();
                    this.Close();
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Saving Settings", "Error on btnOk_Click", ex);
                this.Close();
            }
        }

        private void sldOpacity_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            try
            {
                if (Owner != null)
                {
                    Owner.Opacity = sldOpacity.Value / 100d;
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Changing Opacity", "Error on sldOpacity_ValueChanged", ex);
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                txtLogLocation.Text = ((MainWindow)Owner).LogFolderPath;
                sldOpacity.Value = Owner.Opacity * 100;
                chkAlwaysOnTop.IsChecked = Owner.Topmost;
                chkStreamMode.IsChecked = ((MainWindow)Owner).IsStreamMode;
            }
            catch (Exception ex)
            {
                HandleError("Error Loading Fight", "Error on ddlFights_SelectionChanged", ex);
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                System.Windows.Forms.FolderBrowserDialog dlg = new System.Windows.Forms.FolderBrowserDialog();
                dlg.ShowNewFolderButton = false;
                if (System.IO.Directory.Exists(txtLogLocation.Text))
                    dlg.SelectedPath = txtLogLocation.Text;

                System.Windows.Forms.DialogResult result = dlg.ShowDialog();
                if (result == System.Windows.Forms.DialogResult.OK)
                {
                    txtLogLocation.Text = dlg.SelectedPath;
                }
            }
            catch (Exception ex)
            {
                HandleError("Error Loading Folder Dialog", "Error on Folder Dialog button click", ex);
            }
        }
        private void HandleError(string errorMessage, string logMessage, Exception ex)
        {
            if (!string.IsNullOrEmpty(errorMessage))
            {
                if (ex != null)
                    errorMessage += Environment.NewLine + ex.Message;
                MessageWindow m = new MessageWindow(errorMessage, true, false, true);
                m.Owner = this;
                m.ShowDialog();
            }
            if (!string.IsNullOrEmpty(logMessage))
            {
                if (ex == null)
                    Logger.WriteError(logMessage);
                else
                    Logger.WriteError(logMessage, ex);
            }
        }

        private void btnRestore_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                MainWindow mw =((MainWindow)Owner);
                mw.ResetSettings(false);
                txtLogLocation.Text = mw.LogFolderPath;
                sldOpacity.Value = mw.Opacity * 100;
                chkAlwaysOnTop.IsChecked = mw.Topmost;
                this.Close();
            }
            catch (Exception ex)
            {
                HandleError("Error Loading Folder Dialog", "Error on Folder Dialog button click", ex);
            }
        }
    }
}
