﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ParsecParser;
using ParsecClient.Classes;

namespace ParsecClient
{
    /// <summary>
    /// Interaction logic for StatsOverview.xaml
    /// </summary>
    public partial class StatsOverview : Window
    {
        public MainWindow MainUIWindow;
        public StatsOverview(MainWindow mainUIWindow)
        {
            InitializeComponent();
            MainUIWindow = mainUIWindow;
            CommandBindings.Add(new CommandBinding(ApplicationCommands.Close, new ExecutedRoutedEventHandler(delegate(object sender, ExecutedRoutedEventArgs args) { this.Close(); })));
        }

        public void SetStats(CombatDetails c)
        {
            Topmost = true;
            if (c != null && c.CombatTimeSpan.TotalSeconds > 0)
            {
                lblLengthValue.Text = String.Format("{0:hh\\:mm\\:ss\\.fff}", c.CombatTimeSpan);
                lblApmValue.Text = (c.Activates / c.CombatTimeSpan.TotalMinutes).ToString("#0.0");
                lblDamageOutValue.Text = c.DamageOut.ToString("#,###,###,##0");
                lblDamageOutValue2.Text = (c.DamageOut / c.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblThreatValue.Text = c.Threat.ToString("#,###,###,##0");
                lblThreatValue2.Text = (c.Threat / c.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsOutValue.Text = c.HealOut.ToString("#,###,###,##0");
                lblHealsOutValue2.Text = (c.HealOut / c.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsOutValue3.Text = (c.HealOut > 0 ? ((double)c.EffectiveHealOut / (double)c.HealOut) : 0).ToString("##0.0%");
                lblHealsOutValue4.Text = (c.EffectiveHealOut / c.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblDamageInValue.Text = c.DamageIn.ToString("#,###,###,##0");
                lblDamageInValue2.Text = (c.DamageIn / c.CombatTimeSpan.TotalSeconds).ToString("#,##0");
                lblHealsInValue.Text = c.HealIn.ToString("#,###,###,##0");
                lblHealsInValue2.Text = (c.HealIn / c.CombatTimeSpan.TotalSeconds).ToString("#,##0");
            }
            else
            {
                ClearStats();
            }
        }
        public void ClearStats()
        {
            lblApmValue.Text = "";
            lblThreatValue.Text = "";
            lblLengthValue.Text = "";
            lblDamageOutValue.Text = "";
            lblHealsOutValue.Text = "";
            lblDamageInValue.Text = "";
            lblHealsInValue.Text = "";
            lblDamageOutValue2.Text = "";
            lblHealsOutValue2.Text = "";
            lblDamageInValue2.Text = "";
            lblHealsInValue2.Text = "";
            lblHealsOutValue3.Text = "";
            lblHealsOutValue4.Text = "";
            lblThreatValue2.Text = "";
        }
        public double GetBackgroundOpacity()
        {
            return BorderBackground.Opacity;
        }
        public void SetBackgroundOpacity(double opacity)
        {

            BorderBackground.Opacity = opacity;
            BorderSolid.Opacity = opacity;
        }

        public void DragWindow(object sender, MouseButtonEventArgs args)
        {
            try
            {
                this.WindowState = WindowState.Normal;
                DragMove();
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out DragWindow", this.Title), ex, true);
            }
        }
        private void btnIncreaseOpacity_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (BorderBackground.Opacity < 1)
                {
                    BorderBackground.Opacity += .1;
                    BorderSolid.Opacity += .1;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseOpacity_Click", this.Title), ex, true);
            }
        }

        private void btnDecreaseOpacity_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (BorderBackground.Opacity > 0)
                {
                    BorderBackground.Opacity -= .1;
                    BorderSolid.Opacity -= .1;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnDecreaseOpacity_Click", this.Title), ex, true);
            }
        }

        private void btnIncreaseSize_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.Width < 1000)
                {
                    //this.Height += 10;
                    this.Width += 10;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }

        private void btnDecreaseSize_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.Width > 100)
                {
                    //this.Height -= 10;
                    this.Width -= 10;
                    if (MainUIWindow != null)
                    {
                        MainUIWindow.SaveSettings();
                    }
                }
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (MainUIWindow != null)
                {
                    MainUIWindow.SaveSettings();
                    MainUIWindow.btnStats.IsChecked = false;
                }
                this.Hide();
            }
            catch (Exception ex)
            {
                MainUIWindow.HandleError("Error", string.Format("Error on {0} Pop Out btnIncreaseSize_Click", this.Title), ex, true);
            }
        }
    }
}
