﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ParsecParser
{
    public class CombatDetails : CombatStatistics
    {
        public bool NextFightStarted { get; set; }
        public bool DataHasBeenSynced { get; set; }
        public bool CompanionInFight { get; set; }
        public bool IsPvpFight { get; set; }
        public CombatLogParticipant Companion { get; set; }
        public int CompanionDamageOut { get; set; }
        public int CompanionDamageIn { get; set; }
        public int CompanionAbsorbedIn { get; set; }
        public int CompanionHealOut { get; set; }
        public int CompanionEffectiveHealOut { get; set; }
        public int CompanionHealIn { get; set; }
        public int CompanionThreat { get; set; }
        public List<CombatLogParticipant> Participants { get; set; }
        public Dictionary<long, int> ThreatList { get; set; }
        public int HealThreat { get; set; }
        public List<CombatLogEvent> Events { get; set; }
        public List<CombatStatistics> RaidDetails { get; set; }
        public CombatDetails() : base()
        {
            Participants = new List<CombatLogParticipant>();
            Events = new List<CombatLogEvent>();
            ThreatList = new Dictionary<long, int>();
            RaidDetails = new List<CombatStatistics>();
        }
        public override string ToString()
        {
            return this.ToString(false);
        }
        public string ToString(bool isArchive)
        {
            if (!NextFightStarted && !isArchive)
            {
                return "Current Fight";
            }
            StringBuilder f = new StringBuilder(StartCombat.ToString("HH:mm:ss.fff"));
            if (Participants.Count > 1)
            {
                if (RaidEncounterId > 0 && RaidEncounterId < CombatLogConstants.RaidEncounters.RaidEncounterNames.Length)
                {
                    f.Append(" - ");
                    f.Append(CombatLogConstants.RaidEncounters.RaidEncounterNames[RaidEncounterId]);
                }
                else
                {
                    var parts = Participants.Where(y => !y.IsThisPlayer && y.DisplayName != CombatLogParticipant.EmptyParticipantName).OrderBy(y => y.IsPlayer).ThenBy(y => y.DisplayName).GroupBy(y => y.DisplayNameWithOwner).Select(y => y.Key + (y.Count() > 1 ? "(" + y.Count() + ")" : ""));
                    string p = string.Join(", ", parts.Take(10).ToArray()).TruncateText(120, ", ");
                    if (IsPvpFight)
                        f.Append(" - PvP: ");
                    else
                        f.Append(" - ");
                    f.Append(p);
                }
            }
            return f.ToString();
        }
    }
    [Serializable]
    public class CombatStatistics
    {
        public string CharacterName { get; set; }
        public int Activates { get; set; }
        public int DamageOut { get; set; }
        public int DamageIn { get; set; }
        public int AbsorbedIn { get; set; }
        public int HealOut { get; set; }
        public int EffectiveHealOut { get; set; }
        public int HealIn { get; set; }
        public int Threat { get; set; }
        public DateTime StartCombat { get; set; }
        public DateTime? EndCombat { get; set; }
        public int RaidEncounterId { get; set; }
        public TimeSpan CombatTimeSpan { get; set; }
        public long CombatTicks { get; set; }
        public CombatStatistics()
        {
        }
    }
}
