﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ParsecParser
{
    [Serializable]
    public class CombatLog
    {
        public string CharacterName { get; set; }
        public int RaidUserId { get; set; }
        public Parser Parser { get; set; }
        public int EventCount { get; set; }
        public List<CombatLogEvent> NonFightEvents {get;set;}
        public string FileName { get; set; }
        public DateTime CurrentLogDate { get; set; }
        public bool IsInCombat { get; set; }
        public bool IsGuarded { get; set; }
        public int LastFightIndex;
        //private DateTime? LastEndCombatEventTime = null;
        private DateTime? FloatingEndCombatTime = null;
        public Dictionary<int, CombatDetails> Fights { get; set; }
        public CombatStatistics LogStatistics { get; set; }
        public double CombatReEntryWindow = 1;
        public double StealthReEntryWindow = 20;
        public CombatLog(ParsecParser.Parser parser)
        {
            Parser = parser;
            NonFightEvents = new List<CombatLogEvent>();
            CurrentLogDate = DateTime.Today;
            LastFightIndex = 0;
            Fights = new Dictionary<int, CombatDetails>();
            LogStatistics = new CombatStatistics();
        }
        public CombatLog(ParsecParser.Parser parser, string fileName)
        {
            NonFightEvents = new List<CombatLogEvent>();
            FileName = fileName;
            Parser = parser;
            CurrentLogDate = Parser.ParseDateFromFileName(fileName, false);
            LastFightIndex = 0;
            Fights = new Dictionary<int, CombatDetails>();
            LogStatistics = new CombatStatistics();
        }
        public CombatLog()
        {
            Parser = new Parser();
            NonFightEvents = new List<CombatLogEvent>();
            CurrentLogDate = DateTime.Today;
            LastFightIndex = 0;
            Fights = new Dictionary<int, CombatDetails>();
            LogStatistics = new CombatStatistics();
        }
        public List<CombatLogEvent> AddLines(List<string> lines)
        {
            if (Parser == null)
                Parser = new ParsecParser.Parser();
            List<CombatLogEvent> newEvents = new List<CombatLogEvent>();
            foreach (string s in lines)
            {
                CombatLogEvent ev = Parser.ParseLine(s, CurrentLogDate);
                ev = AddEvent(ev);
                if (ev != null)
                    newEvents.Add(ev);
            }
            return newEvents;
        }
        private DateTime? IgnoreCombatWindowStart = null;
        private DateTime? IgnoreCombatWindowEnd = null;
        private bool IsInCombatIgnoreWindow(DateTime checkTime)
        {
            if (!IgnoreCombatWindowStart.HasValue || !IgnoreCombatWindowEnd.HasValue)
                return false;
            else
                return IgnoreCombatWindowStart.Value.CompareTo(checkTime) <= 0 && IgnoreCombatWindowEnd.Value.CompareTo(checkTime) >= 0;
        }
        public void EndCurrentFight()
        {
            //if we were in combat
            if (IsInCombat && FloatingEndCombatTime.HasValue)
            {
                IsInCombat = false;
                FloatingEndCombatTime = null;
            }
        }
        public CombatLogEvent AddEvent(CombatLogEvent ev)
        {
            if (ev != null)
            {
                EventCount++;
                ev.EventId = EventCount;
                ev.CharacterName = CharacterName;
                CurrentLogDate = ev.TimeStamp;
                if (string.IsNullOrEmpty(CharacterName) && ev.Source.IsPlayer && ev.Target.IsPlayer && !ev.Source.IsPlayerCompanion && !ev.Target.IsPlayerCompanion && (ev.Source.DisplayName == ev.Target.DisplayName))
                {
                    CharacterName = ev.Source.DisplayName;
                }
                ev.Source.SetIsThisPlayer(CharacterName);
                ev.Target.SetIsThisPlayer(CharacterName);
                if (ev.Source.IsThisPlayer && ev.EffectName.EntityId == CombatLogConstants.EffectNames.EnterCombat)
                {
                    bool connectFight = FloatingEndCombatTime.HasValue && ev.TimeStamp.CompareTo(FloatingEndCombatTime.Value) <= 0;
                    if (!connectFight)
                    {
                        if (LastFightIndex > 0)
                            Fights[LastFightIndex].NextFightStarted = true;
                        LastFightIndex++;
                        Fights.Add(LastFightIndex, new CombatDetails() { StartCombat = ev.TimeStamp, CharacterName = CharacterName });
                    }
                    FloatingEndCombatTime = null;
                    IsInCombat = true;
                }
                else if (IsInCombat && FloatingEndCombatTime.HasValue)
                {
                    if (ev.TimeStamp.CompareTo(FloatingEndCombatTime.Value) >= 0)
                    {
                        IsInCombat = false;
                        FloatingEndCombatTime = null;
                    }
                    else if (ev.EffectName != null && ev.Ability != null && ev.EffectName.EntityId == CombatLogConstants.EffectNames.AbilityActivate && ev.Ability.IsCombatDroppedAbility())
                    {
                        //check for stealth if it is detected extend EndCombatTime to 15 seconds and set IgnoreNextEnterCombat
                        FloatingEndCombatTime = ev.TimeStamp.AddSeconds(this.StealthReEntryWindow);
                        IgnoreCombatWindowStart = ev.TimeStamp;
                        IgnoreCombatWindowEnd = FloatingEndCombatTime;

                    }
                }
                //check if guard is applied or removed
                if (ev.Target.IsThisPlayer && ev.EffectType.EntityId == CombatLogConstants.EffectTypes.ApplyEffect && ev.EffectName.EffectIsGuard())
                {
                    IsGuarded = true;
                }
                if (ev.Target.IsThisPlayer && ev.EffectType.EntityId == CombatLogConstants.EffectTypes.RemoveEffect && ev.EffectName.EffectIsGuard())
                {
                    IsGuarded = false;
                }
                if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Heal)
                {
                    ev.EffectiveHeal = ev.Ability.CalculateEffectiveHeal(ev.Value, ev.Threat, IsGuarded);
                }
                if (IsInCombat)
                {
                    ev.FightIndex = LastFightIndex;
                    bool addSource = true;
                    bool addTarget = ev.Source.CompareTo(ev.Target) != 0;
                    CombatDetails currentFight = Fights[LastFightIndex];
                    CombatLogEvent lastFightEvent = currentFight.Events.LastOrDefault();
                    //only add timespan if we are not in the floating 1 sec window
                    if (!FloatingEndCombatTime.HasValue && currentFight.EndCombat.HasValue) //if floating endcombattime is null and endcombat has value reset end combat time
                        currentFight.EndCombat = null;
                    if (!currentFight.EndCombat.HasValue)//if endcombat doesnt have a value keep time running
                    {
                        if (lastFightEvent != null)
                            LogStatistics.CombatTimeSpan = LogStatistics.CombatTimeSpan.Add(ev.TimeStamp.Subtract(lastFightEvent.TimeStamp));
                        currentFight.CombatTimeSpan = ev.TimeStamp.Subtract(currentFight.StartCombat);
                    }
                    foreach (CombatLogParticipant clp in currentFight.Participants)
                    {
                        if (addSource && ev.Source.CompareTo(clp) == 0)
                            addSource = false;
                        if (addTarget && ev.Target.CompareTo(clp) == 0)
                            addTarget = false;
                        if (!addTarget && !addSource) //leave early if both are found
                            break;
                    }
                    if (addTarget)
                    {
                        currentFight.Participants.Add(ev.Target);
                        if (currentFight.RaidEncounterId == 0)
                            currentFight.RaidEncounterId = ev.Target.GetRaidEncounterId();
                    }
                    if (addSource)
                    {
                        currentFight.Participants.Add(ev.Source);
                        if (currentFight.RaidEncounterId == 0)
                            currentFight.RaidEncounterId = ev.Source.GetRaidEncounterId();
                    }
                    if (ev.Target.IsThisPlayer && (ev.EffectName.EntityId == CombatLogConstants.EffectNames.ExitCombat || ev.EffectName.EntityId == CombatLogConstants.EffectNames.Death || ev.EffectName.EntityId == CombatLogConstants.EffectNames.SafeLoginImmunity))
                    {
                        //LastEndCombatEventTime = ev.TimeStamp;
                        currentFight.EndCombat = ev.TimeStamp;
                        if (IsInCombatIgnoreWindow(ev.TimeStamp))
                            FloatingEndCombatTime = IgnoreCombatWindowEnd;
                        else
                        {
                            FloatingEndCombatTime = ev.TimeStamp.AddSeconds(this.CombatReEntryWindow);                            
                            IgnoreCombatWindowStart = ev.TimeStamp;
                            IgnoreCombatWindowEnd = FloatingEndCombatTime;
                        }
                    }
                    //else if (currentFight.EndCombat.HasValue)
                    //{
                    //    currentFight.EndCombat = ev.TimeStamp;
                    //}
                    if (ev.EffectName != null)
                    {
                        if (ev.Source.IsThisPlayer)
                        {
                            if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Damage)
                            {
                                currentFight.DamageOut += ev.Value;
                                LogStatistics.DamageOut += ev.Value;
                                if (ev.Threat != 0)
                                {
                                    int effectiveThreat = CombatLogHelpers.GetEffectiveThreat(currentFight.Threat, ev.Threat);
                                    currentFight.Threat += effectiveThreat;
                                    LogStatistics.Threat += effectiveThreat;
                                    if (!currentFight.ThreatList.ContainsKey(ev.Target.UniqueId))
                                        currentFight.ThreatList.Add(ev.Target.UniqueId, CombatLogHelpers.GetEffectiveThreat(0, ev.Threat)); //add enemy to threat list
                                    else
                                        currentFight.ThreatList[ev.Target.UniqueId] += CombatLogHelpers.GetEffectiveThreat(currentFight.ThreatList[ev.Target.UniqueId], ev.Threat); //update threat
                                }
                                if (!currentFight.IsPvpFight && ev.Source.IsPlayer && !ev.Source.IsPlayerCompanion && ev.Target.IsPlayer && !ev.Target.IsPlayerCompanion && ev.Target.CompareTo(ev.Source) != 0)
                                {
                                    currentFight.IsPvpFight = true; //mark as pvp fight if event is damage and source and target are players
                                }
                            }
                            else if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Heal)
                            {
                                if (ev.Target.IsPlayer || ev.Target.IsPlayerCompanion) //only count heals that heal players or companions
                                {
                                    currentFight.HealOut += ev.Value;
                                    currentFight.EffectiveHealOut += ev.EffectiveHeal;
                                    LogStatistics.HealOut += ev.Value;
                                    LogStatistics.EffectiveHealOut += ev.EffectiveHeal;
                                }
                                if (ev.Threat != 0)
                                {
                                    int effectiveThreat = CombatLogHelpers.GetEffectiveThreat(currentFight.Threat, ev.Threat);
                                    currentFight.Threat += effectiveThreat;
                                    LogStatistics.Threat += effectiveThreat;
                                    var keys = new List<long>(currentFight.ThreatList.Keys);
                                    foreach (long i in keys)
                                    {
                                        currentFight.ThreatList[i] += CombatLogHelpers.GetEffectiveThreat(currentFight.ThreatList[i], ev.Threat);
                                    }
                                    currentFight.HealThreat += CombatLogHelpers.GetEffectiveThreat(currentFight.HealThreat, ev.Threat);
                                }
                            }
                            else
                            {
                                if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.AbilityActivate)
                                {
                                    currentFight.Activates++;
                                    LogStatistics.Activates++;
                                }
                                if (ev.Threat != 0) //if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.ModifyThreat || ev.EffectName.EntityId == CombatLogConstants.EffectNames.Taunt)
                                {
                                    int effectiveThreat = CombatLogHelpers.GetEffectiveThreat(currentFight.Threat, ev.Threat);
                                    currentFight.Threat += effectiveThreat;
                                    LogStatistics.Threat += effectiveThreat;

                                    if (!currentFight.ThreatList.ContainsKey(ev.Target.UniqueId))
                                        currentFight.ThreatList.Add(ev.Target.UniqueId, CombatLogHelpers.GetEffectiveThreat(0, ev.Threat)); //add enemy to threat list
                                    else
                                        currentFight.ThreatList[ev.Target.UniqueId] += CombatLogHelpers.GetEffectiveThreat(currentFight.ThreatList[ev.Target.UniqueId], ev.Threat); //update threat
                                }
                            }
                        }
                        if (ev.Target.IsThisPlayer)
                        {
                            if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Damage)
                            {
                                currentFight.DamageIn += ev.Value;
                                LogStatistics.DamageIn += ev.Value;
                                currentFight.AbsorbedIn += ev.AbsorbedValue;
                                LogStatistics.AbsorbedIn += ev.AbsorbedValue;
                            }
                            else if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Heal)
                            {
                                currentFight.HealIn += ev.Value;
                                LogStatistics.HealIn += ev.Value;
                            }
                        }
                        if (ev.Source.IsPlayerCompanion && ev.Source.CompanionOwner == CharacterName)
                        {
                            currentFight.CompanionInFight = true;
                            currentFight.Companion = ev.Source;
                            //companion is source
                            if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Damage)
                            {
                                currentFight.CompanionDamageOut += ev.Value;
                            }
                            else if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Heal)
                            {
                                currentFight.CompanionHealOut += ev.Value;
                                currentFight.CompanionEffectiveHealOut += ev.EffectiveHeal;
                            }

                            if (ev.Threat != 0)
                            {
                                int effectiveThreat = CombatLogHelpers.GetEffectiveThreat(currentFight.Threat, ev.Threat);
                                currentFight.CompanionThreat += effectiveThreat;
                            }
                        }
                        if (ev.Target.IsPlayerCompanion && ev.Target.CompanionOwner == CharacterName)
                        {
                            currentFight.CompanionInFight = true;
                            currentFight.Companion = ev.Target;
                            //companion is target
                            if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Damage)
                            {
                                currentFight.CompanionDamageIn += ev.Value;
                                currentFight.CompanionAbsorbedIn += ev.AbsorbedValue;
                            }
                            else if (ev.EffectName.EntityId == CombatLogConstants.EffectNames.Heal)
                            {
                                currentFight.CompanionHealIn += ev.Value;
                            }
                        }
                        currentFight.Events.Add(ev);
                    }
                }
                else
                {
                    NonFightEvents.Add(ev);
                }
                return ev;
            }
            return null;
        }
    }
}
