﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace ParsecParser
{
    public static class CombatLogHelpers
    {
        public static string XmlRegexPattern = "<[^<>]+>";
        public static string DigitsOrLetters = "^[a-zA-Z0-9]*$";
        public static string DigitsOrLettersOrSpaces = "^[a-zA-Z0-9]*[a-zA-Z0-9 ]*[a-zA-Z0-9]$";
        public static bool ContainsString(this string s1, string s2, bool skipHtml)
        {
            s1 = s1.NullToEmpty();
            s2 = s2.NullToEmpty();
            int resultStartIndex = s1.IndexOf(s2, StringComparison.OrdinalIgnoreCase);
            bool isFound = resultStartIndex >= 0;
            if (skipHtml)
            {
                Regex r = new Regex(XmlRegexPattern);
                string taglessS1 = r.Replace(s1, "");
                return taglessS1.IndexOf(s2, StringComparison.OrdinalIgnoreCase) >= 0;
            }
            else
                return isFound;
        }
        public static bool IsDigitsOrLetters(this string s)
        {
            Regex r = new Regex(DigitsOrLetters);
            return r.IsMatch(s);
        }
        public static bool IsDigitsOrLettersOrSpaces(this string s)
        {
            Regex r = new Regex(DigitsOrLettersOrSpaces);
            return r.IsMatch(s);
        }
        public static string NullToEmpty(this string source)
        {
            return source ?? string.Empty;
        }
        public static string EmptyToNull(this string source)
        {
            return source == string.Empty ? null : source;
        }
        public static string EncodeJsString(this string s)
        {
            if (s == null)
                s = string.Empty;
            StringBuilder sb = new StringBuilder();
            sb.Append("\"");
            foreach (char c in s)
            {
                switch (c)
                {
                    case '\"':
                        sb.Append("\\\"");
                        break;
                    case '\\':
                        sb.Append("\\\\");
                        break;
                    case '\b':
                        sb.Append("\\b");
                        break;
                    case '\f':
                        sb.Append("\\f");
                        break;
                    case '\n':
                        sb.Append("\\n");
                        break;
                    case '\r':
                        sb.Append("\\r");
                        break;
                    case '\t':
                        sb.Append("\\t");
                        break;
                    default:
                        int i = (int)c;
                        if (i < 32 || i > 127)
                        {
                            sb.AppendFormat("\\u{0:X04}", i);
                        }
                        else
                        {
                            sb.Append(c);
                        }
                        break;
                }
            }
            sb.Append("\"");

            return sb.ToString();
        }
        public static double GetThreatCoefficient(this CombatLogEntity ability, bool isGuarded)
        {
            switch (ability.EntityId)
            {
                case CombatLogConstants.Abilities.ImpeccableMedpacc:
                    return isGuarded ? .375 : .5;
                case CombatLogConstants.Abilities.PsychicSuffusion:
                    return isGuarded ? .375 : .5;
                case CombatLogConstants.Abilities.PreventativeMedicine:
                    return isGuarded ? .375 : .5;
                case CombatLogConstants.Abilities.PsychAid:
                    return isGuarded ? .375 : .5;
                case CombatLogConstants.Abilities.ForceArmor:
                    return isGuarded ? .375 : .5;
                case CombatLogConstants.Abilities.ForceBarrier:
                    return isGuarded ? .375 : .5;
                case CombatLogConstants.Abilities.AdrenalineRush:
                    return isGuarded ? .375 : .5;
                default:
                    return isGuarded ? .325 : .45;
            }
        }
        public static int CalculateEffectiveHeal(this CombatLogEntity ability, int amount, int threat, bool isGuarded)
        {
            if (threat == 0)
                return 0;
            else
            {
                int effectiveHeal = (int)Math.Ceiling((double)threat / ability.GetThreatCoefficient(isGuarded));
                if (effectiveHeal > amount)
                    return amount;
                else
                    return effectiveHeal;
            }
        }
        public static bool EffectIsGuard(this CombatLogEntity effect)
        {
            switch (effect.EntityId)
            {
                case CombatLogConstants.Guards.AssassinGuard:
                case CombatLogConstants.Guards.BountyHunterGuard:
                case CombatLogConstants.Guards.GuardianGuard:
                case CombatLogConstants.Guards.JuggernautGuard:
                case CombatLogConstants.Guards.ShadowGuard:
                case CombatLogConstants.Guards.VanguardGuard:
                    return true;
                default:
                    return false;
            }
        }
        public static bool IsCombatDroppedAbility(this CombatLogEntity ability)
        {
            switch (ability.EntityId)
            {
                case CombatLogConstants.CombatDropAbilities.DisappearingAct:
                case CombatLogConstants.CombatDropAbilities.ShadowForceCloak:
                case CombatLogConstants.CombatDropAbilities.AssassinForceCloak:
                case CombatLogConstants.CombatDropAbilities.CloakingScreen:
                    return true;
                default:
                    return false;
            }
        }
        public static bool MitigationIsDefense(this CombatLogEntity mitigation)
        {
            switch (mitigation.EntityId)
            {
                case CombatLogConstants.Mitigation.Dodge:
                case CombatLogConstants.Mitigation.Deflect:
                case CombatLogConstants.Mitigation.Miss:
                case CombatLogConstants.Mitigation.Parry:
                    return true;
                default:
                    return false;
            }
        }
        public static bool IsShieldAbility(this CombatLogEntity ability)
        {
            switch (ability.EntityId)
            {
                case CombatLogConstants.ShieldAbilities.ForceArmor:
                case CombatLogConstants.ShieldAbilities.StaticBarrier:
                    return true;
                default:
                    return false;
            }
        }
        public static bool IsShieldEffect(this CombatLogEntity effect)
        {
            switch (effect.EntityId)
            {
                case CombatLogConstants.ShieldAbilities.BladeBarrier:
                case CombatLogConstants.ShieldAbilities.SonicBarrier:
                    return true;
                default:
                    return false;
            }
        }
        public static string TruncateText(this string source, int characterCount, string truncateAtLastIndexOf)
        {
            if (!string.IsNullOrEmpty(truncateAtLastIndexOf))
            {
                if (source.Length + 3 > characterCount)
                {
                    int indexOfTruncate = source.LastIndexOf(truncateAtLastIndexOf);
                    while (indexOfTruncate > characterCount)
                    {
                        indexOfTruncate = source.LastIndexOf(truncateAtLastIndexOf, indexOfTruncate - 1);
                    }
                    if (indexOfTruncate > 0)                    
                        return source.TruncateText(indexOfTruncate+3, true);
                }
            }
            return source.TruncateText(characterCount, true);
        }
        public static string TruncateText(this string source, int characterCount, bool truncateAtNewline)
        {
            string Elipsis = "...";
            return source.TruncateText(characterCount, truncateAtNewline, Elipsis);
        }
        public static string TruncateText(this string source, int characterCount, bool truncateAtNewline, string postFix)
        {
            if (string.IsNullOrEmpty(source))
                return source;
            else if (characterCount <= postFix.Length)
                return postFix;
            else
            {
                int truncatedLength = source.Length;
                int indexOfFirstNewLine = source.IndexOf(Environment.NewLine, 0);
                if (truncateAtNewline && indexOfFirstNewLine > 0)
                    truncatedLength = indexOfFirstNewLine + 1;
                if (truncatedLength > characterCount)
                    truncatedLength = characterCount - postFix.Length;
                if (truncatedLength == source.Length)
                    return source;
                else
                    return source.Substring(0, truncatedLength) + postFix;
            }
        }
        public static int GetEffectiveThreat(int currentThreat, int newThreat)
        {

            if (newThreat < 0 && Math.Abs(newThreat) > currentThreat)
                return -currentThreat;
            else
                return newThreat;
        }
        public static int GetRaidEncounterId(this CombatLogParticipant p)
        {
            switch (p.EntityId)
            {
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.AnnihilationDroidXRR3:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Sixteen.AnnihilationDroidXRR3:
                    return 1;
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Gharj:
                    return 2;
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Puzzle_ProtoAcklay:
                    return 3;
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_JaelTheSilencer:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_KaheshTheReborn:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_AlariiSisterOfSorrow:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_FahrenJajTheLaughingLord:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_LuthroTheKinslayer:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_SerrodTheMessenger:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_YshaarKaelBetrayerOfAdas:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Council_ZaheenKaurTheBowedLord:
                    return 4;
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.BallLightning:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.MindTrap:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Eight.Soa:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Sixteen.BallLightning:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Sixteen.MindTrap:
                case CombatLogConstants.RaidEncounters.EternalVault.HardMode.Sixteen.Soa:
                    return 5;
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.Bonethrasher:
                    return 6;
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.Sorno:
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.Jarg:
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.CarbonizerProbe:
                    return 7;
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.ForemanCrusher:
                    return 8;
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.SecurityStunDrone:
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.ProximityPulseMine:
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.G4B3HeavyFabricator:
                    return 9;
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.ExplodingMouseDroid:
                case CombatLogConstants.RaidEncounters.KaraggasPalace.HardMode.Eight.KaraggaTheUnyielding:
                    return 10;
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.Toth:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.Zorn:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.HandlerMurdok:
                    return 11;
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.FirebrandBattleTank:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.StormcallerBlastTank:
                    return 12;
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.ColonelVorgath:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.AutomatedDefenseTurret:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.ImperialDemolitionsProbe:
                    return 13;
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.ImperialSiegeDroid:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.PulsarPowerDroid:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.TrandoshanTrenchcutter:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.TrandoshanWarrior:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.WarstriderBattlewalker:
                case CombatLogConstants.RaidEncounters.ExplosiveConflict.HardMode.Eight.WarlordKephess:
                    return 14;
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.FoulOffspring:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.JealousMale:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.TwistedSpawn:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.TheWrithingHorror:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.FoulOffspring:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.JealousMale:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.TheWrithingHorror:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.FoulOffspring:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.JealousMale:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.TwistedSpawn:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.TheWrithingHorror:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.FoulOffspring:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.JealousMale:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.TheWrithingHorror:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.TheWrithingHorror_Other:
                    return 15;
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.Kelsara:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.Ciphas:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.Heirad:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.Kelsara:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.Ciphas:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.Heirad:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.Kelsara:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.Ciphas:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.Heirad:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.Kelsara:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.Ciphas:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.Heirad:
                    return 16;
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.MasterControl:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.OperatorIX:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.Regulator:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.MasterControl:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.OperatorIX:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.Regulator:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.MasterControl:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.OperatorIX:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.Regulator:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.MasterControl:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.OperatorIX:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.Regulator:
                    return 17;
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.KephessTheUndying:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.KephessTheUndying:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.KephessTheUndying:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.KephessTheUndying:
                    return 18;
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.GraspingTentacle:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.TunnelingTentacleA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.TunnelingTentacleB:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.HypergateIrregularity:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.TheTerrorFromBeyondA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Eight.TheTerrorFromBeyondB:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.GraspingTentacle:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.TunnelingTentacleA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.TunnelingTentacleB:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.HypergateIrregularity:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.TheTerrorFromBeyondA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.StoryMode.Sixteen.TheTerrorFromBeyondB:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.GraspingTentacle:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.TunnelingTentacleA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.TunnelingTentacleB:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.HypergateIrregularity:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.TheTerrorFromBeyondA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Eight.TheTerrorFromBeyondB:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.GraspingTentacle:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.TunnelingTentacleA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.TunnelingTentacleB:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.HypergateIrregularity:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.TheTerrorFromBeyondA:
                case CombatLogConstants.RaidEncounters.TerrorFromBeyond.HardMode.Sixteen.TheTerrorFromBeyondB:
                    return 19;
                case CombatLogConstants.RaidEncounters.ToborrosCourtyard.StoryMode.Eight.GoldenFury:
                case CombatLogConstants.RaidEncounters.ToborrosCourtyard.StoryMode.Sixteen.GoldenFury:
                case CombatLogConstants.RaidEncounters.ToborrosCourtyard.HardMode.Eight.GoldenFury:
                case CombatLogConstants.RaidEncounters.ToborrosCourtyard.HardMode.Sixteen.GoldenFury:
                case CombatLogConstants.RaidEncounters.ToborrosCourtyard.PalaceSecurityDroid:
                case CombatLogConstants.RaidEncounters.ToborrosCourtyard.Isotope5A:
                case CombatLogConstants.RaidEncounters.ToborrosCourtyard.Isotope5B:
                    return 20;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Eight.DashRoode:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.DashRoode:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.DashRoode:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.DashRoode:
                    return 21;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Eight.Titan6:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.Titan6:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.Titan6:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.Titan6:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.TitanProbe:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.RockShelter:
                    return 22;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Eight.Thrasher:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.Thrasher:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.Thrasher:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.Thrasher:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.MercenarySniper:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.MercenaryDemolitionist:
                    return 23;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.RedTeamAssaultGunner:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.RedTeamFlametech:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.BlueTeamPackHunterA:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.BlueTeamPackHunterB:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.GreenTeamDuelist:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.GreenTeamPyroguard:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.GoldTeamMedtech:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.GoldTeamPyroguard:
                    return 24;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Eight.OperationsChief:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.OperationsChief:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.OperationsChief:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.OperationsChief:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.RailTurret:
                    return 25;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.Bodyguard:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.WealthyBuyer:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.P2BAssaultDroid_1:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.P2BAssaultDroid_2:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.IA82ArtilleryDroid_1:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.MA9FrontlineDroid_1:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.MA9FrontlineDroid_2:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.PX7ReconDroid_1:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.PX7ReconDroid_2:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.OlokTheShadow_Other:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Eight.OlokTheShadow:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.OlokTheShadow:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.OlokTheShadow:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.OlokTheShadow:
                    return 26;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Eight.Tuchuk:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.Tuchuk:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.Tuchuk:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.Tuchuk:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.VilusGarr:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.CaptainHoric:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.Sunder:
                    return 27;
                case CombatLogConstants.RaidEncounters.ScumAndVillany.DreadGuard:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.DreadGuardOfficer:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.KellDragonA:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.KellDragonB:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.DreadMasterStyrak_Other1:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.DreadMasterStyrak_Other2:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Eight.DreadMasterStyrak:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.DreadMasterStyrakA:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.DreadMasterStyrakB:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.StoryMode.Sixteen.DreadMasterStyrakC:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.DreadMasterStyrakA:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.DreadMasterStyrakB:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.DreadMasterStyrakC:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Eight.DreadMasterStyrakD:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.DreadMasterStyrakA:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.DreadMasterStyrakB:
                case CombatLogConstants.RaidEncounters.ScumAndVillany.HardMode.Sixteen.DreadMasterStyrakC:
                    return 28;
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.NefraWhoBlocksTheWay:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Sixteen.NefraWhoBlocksTheWay:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Eight.NefraWhoBlocksTheWay:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Sixteen.NefraWhoBlocksTheWay:
                    return 29;
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.GateCommanderDraxus:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Sixteen.GateCommanderDraxus:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Eight.GateCommanderDraxus:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Sixteen.GateCommanderDraxus:
                    return 30;
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.GrobThrokWhoFeedsTheForge:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Sixteen.GrobThrokWhoFeedsTheForge:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Eight.GrobThrokWhoFeedsTheForge:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Sixteen.GrobThrokWhoFeedsTheForge:
                    return 31;
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.CorruptorZero:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Sixteen.CorruptorZero:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Eight.CorruptorZero:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Sixteen.CorruptorZero:
                    return 32;
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.DreadMasterBrontes1:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.DreadMasterBrontes2:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.HandOfBrontes1:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Eight.HandOfBrontes2:
                case CombatLogConstants.RaidEncounters.DreadFortress.HardMode.Sixteen.DreadMasterBrontes:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Eight.DreadMasterBrontes:
                case CombatLogConstants.RaidEncounters.DreadFortress.StoryMode.Sixteen.DreadMasterBrontes:                
                case CombatLogConstants.RaidEncounters.DreadFortress.FingerOfBrontes1:
                case CombatLogConstants.RaidEncounters.DreadFortress.FingerOfBrontes2:
                case CombatLogConstants.RaidEncounters.DreadFortress.FingerOfBrontes3:
                case CombatLogConstants.RaidEncounters.DreadFortress.LeftHandOfBrontes:
                case CombatLogConstants.RaidEncounters.DreadFortress.RightHandOfBrontes:
                case CombatLogConstants.RaidEncounters.DreadFortress.HandOfBrontes:
                case CombatLogConstants.RaidEncounters.DreadFortress.CorruptedClone:
                    return 33;
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadMasterBestia:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadMasterBestia:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadMasterBestia:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadMasterBestia:
                    return 34;
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadMasterTyrans:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadMasterTyrans:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadMasterTyrans:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadMasterTyrans:
                    return 35;
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadMasterCalphayus:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadMasterCalphayus:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.FocusCrystal:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadMasterCalphayus:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadMasterCalphayus1:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadMasterCalphayus2:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadMasterCalphayus3:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadMasterCalphayus4:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.FocusCrystal:
                case CombatLogConstants.RaidEncounters.DreadPalace.DarkGrowth:
                    return 36;
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadMasterRaptus:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadMasterRaptus:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadMasterRaptus:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadMasterRaptus:
                    return 37;
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadCouncilBestia:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadCouncilTyrans:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadCouncilCalphayus:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadCouncilRaptus:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadCouncilBrontes:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Eight.DreadCouncilStyrak:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadCouncilBestia:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadCouncilTyrans:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadCouncilCalphayus:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadCouncilRaptus:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadCouncilBrontes:
                case CombatLogConstants.RaidEncounters.DreadPalace.HardMode.Sixteen.DreadCouncilStyrak:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadCouncilBestia:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadCouncilTyrans:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadCouncilCalphayus:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadCouncilRaptus:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadCouncilBrontes:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Eight.DreadCouncilStyrak:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadCouncilBestia:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadCouncilTyrans:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadCouncilCalphayus:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadCouncilRaptus:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadCouncilBrontes:
                case CombatLogConstants.RaidEncounters.DreadPalace.StoryMode.Sixteen.DreadCouncilStyrak:
                    return 38;
                default:
                    return 0;
            }
        }
    }
}
